$debug

if not LuaOrb_cfg.irep then
   LuaOrb_cfg.irep = createproxy{interface = "CORBA::Repository",
                                 ior_file = "../ir.ref"}
end

dofile ("../console.lua")

tst = createproxy{interface="Tst_All::Tst", ior_file="server.ref"}

print("---------------------")
tstAttrib = tst:getTstAttrib()
assert(tstAttrib.domain == "inf.puc-rio.br")
tstAttrib.host = "obaluae"
assert(tstAttrib.host == "obaluae")
print ("Attributes OK")

tst:print("---------------------")
tst:print(tstAttrib.host.."."..tstAttrib.domain)

print("---------------------")
tstStructSeq = tst:getTstStructSeq()
assert (tstStructSeq:add_book({title="T1", author="A1"}))
assert (not tstStructSeq:add_book({title="T1", author="A1"})) 


booklist = {
   {title="T2", author="A2"},
   {title="T3", author="A3"},
   {title="T4", author="A4"},
   {title="T5", author="A1"},
   {title="T6", author="A1"},
   {title="T7", author="A2"},
   {title="T8", author="A3"},
}

assert (tstStructSeq:add_books(booklist)) 


tinsert(booklist,1,{title="T1", author="A1"})

assert (isSimilar(tstStructSeq:get_allbooks(),booklist))

assert (isEqual(tstStructSeq:get_book("T3"),{title="T3", author="A3"}))

local flag, list = tstStructSeq:get_booklist("A1")

assert (isSimilar(list,{{title="T1", author="A1"},{title="T5", author="A1"},{title="T6", author="A1"}}))

local flag, list = tstStructSeq:get_booklist("A10")
assert (not flag)

print("Structs OK")


print("---------------------")
tstStructArray = tst:getTstStructArray()

p1 = {x=10,y=20}
p2 = {x=30,y=40}


assert (isEqual (tstStructArray:draw_pt(p1), p1))
assert (isEqual (tstStructArray:draw_pt(p2), p2))

rec = tstStructArray:create_rect(p1,p2)
assert (isEqual (tstStructArray:draw_rect(rec), rec))

p3 = {x=15,y=60}
tri = tstStructArray:create_triangle(p1,p2,p3)
assert (isEqual (tstStructArray:draw_triangle(tri),tri))

a = tstStructArray:swap({x=16,y=7})
assert (isEqual(a,{x=7,y=16}))

print("Arrays OK")


print("---------------------")
tstEnum = tst:getTstEnum()
assert (tstEnum:tomorrow("sun") == "mon")
assert (tstEnum:yesterday("sun") == "sat")

assert (tstEnum:tomorrow(4) == "fri")
assert (tstEnum:yesterday(3) == "tue")

print("Enums OK")

-- Auxiliary function

function table2str(table)
   local i,v = next(table,nil)
   local result = "{\n"
   while i do
      if type(i) == "string" then
         result = result .. "   " .. i .. " = "
      else
         result = result .. "   [" .. i .. "] = "
      end
      if type(v) == "table" then
         result = result .. table2str(v)
      elseif type(v) == "number" then
         result = result .. v
      else
         result = result .. "[[" .. v .."]]"
      end
      i,v = next(table,i)
      if i then
         result = result .. ",\n"
      else
         result = result .. "}"
      end
   end
   return result
end



tstException = tst:getTstException()

function tstException:exception_handler(op, ex_name, ex_descr)
   return "exception"
end


print("---------------------")
assert (tstException:tst_point({x=16,y=7}, {p1={x=0,y=0},p2={x=100,y=100}}))
assert (tstException:tst_point({x=16,y=7}, {p1={x=0,y=0},p2={x=10,y=10}}) == "exception")


assert (tstException:sqrt(49) == 7)
assert (tstException:sqrt(-2) == "exception")
assert (format("%1.4f",tstException:sqrt(2)) == format("%1.4f",sqrt(2)))

print("Exceptions OK")


print("---------------------")
tstUnion = tst:getTstUnion()

tstUnion:setField("",200)
assert(tstUnion:getField() == 200)
print("Union short 1 OK")

tstUnion:setField("m_short",{union_field = "m_short", union_value = 200})
assert(tstUnion:getField() == 200)
print("Union short 2 OK")

tstUnion:setField("m_ushort",{union_field = "m_ushort", union_value = 200})
assert(tstUnion:getField() == 200)
print("Union ushort OK")

tstUnion:setField("m_long",{union_field = "m_long", union_value = 150000})
assert(tstUnion:getField() == 150000)
print("Union long OK")

tstUnion:setField("m_ulong",{union_field = "m_ulong", union_value = 150000})
assert(tstUnion:getField() == 150000)
print("Union ulong OK")

tstUnion:setField("m_float",{union_field = "m_float", union_value = 150.34})
assert(format("%3.2f",tstUnion:getField()) == format("%3.2f",150.34))
print("Union float OK")

tstUnion:setField("m_double",{union_field = "m_double", union_value = 3788880000})
assert(format("%g",tstUnion:getField()) == format("%g",3788880000))
print("Union double OK")

tstUnion:setField("m_octet",{union_field = "m_octet", union_value = 8})
assert(tstUnion:getField() == 8)
print("Union octet OK")

tstUnion:setField("","e")
assert(tstUnion:getField() == "e")
print("Union char OK")

tstUnion:setField("m_char",{union_field = "m_char", union_value = "a"})
assert(tstUnion:getField() == "a")
print("Union char 2 OK")

tstUnion:setField("m_string",{union_field = "m_string", union_value = "hello"})
assert(tstUnion:getField() == "hello")
print("Union string OK")

--local value = 
--tstUnion:setField("m_typecode",{union_field = "m_typecode", union_value = value})
--assert(tstUnion:getField() == value)
--print("Union typecode OK")

local value = "mon"
tstUnion:setField("day",{union_field = "day", union_value = value})
assert(tstUnion:getField() == value)
print("Union enum OK")

local value = {title="T2", author= "A2"}
tstUnion:setField("the_book",{union_field = "the_book", union_value = value})
assert(isEqual(tstUnion:getField(), value))
print("Union struct 1 OK")

local value = {x=10,y=20}
tstUnion:setField("a_point",{union_field = "a_point", union_value = value})
assert(isEqual(tstUnion:getField(), value))
print("Union struct 2 OK")

tstUnion:setField("a_rect",{union_field = "a_rect", union_value = rec})
assert(isEqual(tstUnion:getField(), rec))
print("Union struct 3 OK")

tstUnion:setField("books",{union_field = "books", union_value = booklist})
assert(isEqual(tstUnion:getField(), booklist))
print("Union sequence OK")

tstUnion:setField("points",{union_field = "points", union_value = tri})
assert(isEqual(tstUnion:getField(), tri))
print("Union array OK")

tstUnion:setField("m_boolean",{union_field = "m_boolean", union_value = 1})
assert(tstUnion:getField() == 1)
print("Union boolean OK")

tstUnion:setField("m_any",{union_field = "m_any", union_value = "Anything"})
assert(tstUnion:getField() == "Anything")
print("Union any OK")



print("---------------------")
tstAny = tst:getTstAny()

tstAny:setField("",200)
assert(tstAny:getField() == 200)
print("Any number OK")

tstAny:setField("","hello")
assert(tstAny:getField() == "hello")
print("Any string 1 OK")

tstAny:setField("","hello")
assert(tstAny:getField() == "hello")
print("Any string 2 OK")

--tstAny:setField("",tstAttrib)
--assert(tstAny:getField().domain == "inf.puc-rio.br")
--print("Any proxy OK")

exit()
