$debug

if not LuaOrb_cfg.irep then
   LuaOrb_cfg.irep = createproxy{interface = "CORBA::Repository",
                                 ior_file = "../ir.ref"}
end

LuaOrb_cfg.enum_as_numbers = "numeric"

tst_impl = {

   print = function (self, msg)
      print(msg)
   end,


   getTstAttrib = function (self)
      return {
         domain = "inf.puc-rio.br"
      }
   end,
   

   getTstStructSeq = function (self)
      return {
         books = {},

         add_book = function(self, book)
            if not self.books[book.title] then
               self.books[book.title] = book.author
               return 1
            else
               return nil
            end
         end,
         
         add_books = function(self,list)
            local todos = 1
            i = 1
            while i <= list.n do
              if not self:add_book(list[i]) then
                  todos = nil          
               end
               i = i+1
            end
            if todos then return 1 else return nil end
         end,

         get_allbooks = function(self)
            local list = {}
            local i = 0
            local index, value = next(self.books,nil)
            while index do
               i = i + 1
               list[i] = {title=index, author=value}
               index, value = next(self.books,index)
           end
           return list
         end,
         
         get_book = function(self, title)
            if self.books[title] then
               return {title = title, author = self.books[title]}
            else
               return {title="ERROR", author="ERROR"}
            end
         end,

         get_booklist = function(self, author)
            local list = {}
            local i,v = next(self.books,nil)
            local j = 0
            while i do
               if v == author then
                  j = j + 1
                  list[j] = {title=i, author = v}
               end
               i,v = next(self.books,i)
            end
            return (j~=0), list
         end,
      }
   end,


   getTstStructArray = function(self)
      return {
         create_rect = function(self, pt1, pt2)
            return {p1=pt1,p2=pt2}
         end,

         create_triangle = function(self, pt1, pt2, pt3)
            return {pt1,pt2,pt3}
         end,

         draw_pt = function(self,pt)
            return pt
         end,

         draw_rect = function(self,rect)
            return rect
         end,

         draw_triangle = function(self,tri)
            return tri
         end,

         swap = function(self, pt)
            return {x=pt.y, y=pt.x}
         end,
      }
   end,


   getTstEnum = function (self)
      return {
         tomorrow = function (self, day)
            return mod(day+1,7)
         end,

         yesterday = function (self, day)
            return mod(day+6,7)
         end,
      }
   end,

   getTstException = function (self)
      return {
         tst_point = function(self, pt, r)
            if pt.x >= r.p1.x and pt.x <= r.p2.x and
               pt.y >= r.p1.y and pt.y <= r.p2.y then
               return 1
            else
               return LuaOrbException{"InvalidPoint";p=pt,r=r}
            end
         end,

         sqrt = function(self, number)
            if number >= 0 then
               return sqrt(number)
            else
               return LuaOrbException{"OutOfRange"}
            end
         end,
      }
   end,


   getTstUnion = function (self)
      return {
         getField = function (self)
            if self.field then
               return {union_field = self.field, union_value = self.value}
            else
               return self.value
            end
         end,

         setField = function (self, field, value)
            if field == "" then 
               self.field = nil
            else
               self.field = field
            end
            self.value = value
         end,
      }
   end,

   getTstAny = function (self)
      return {
         any = {},
         getField = function (self)
            if self.field then
               return {typecode = self.field, value = self.value}
            else
               return self.value
            end
         end,
         
         setField = function (self, field, value)
            if field == "" then 
               self.field = nil
            else
               self.field = field
            end
            self.value = value
         end,
      }
   end,
}
            



tst_server = CreateDSIServer("Tst_All::Tst",tst_impl)

writeto("server.ref")
write(tst_server:_get_ior())
writeto()
