$debug

--
-- Generic Configuration Functions
--

function saveCache()
   local i,v = next(options,nil)
   writeto(CACHE_FILE)
   write("options = {\n")
   while i do
      if type(v) == "number" then
         write("   "..i.." = "..v..",\n")
      elseif type(v) == "string" then
         write("   "..i.." = [["..v.."]],\n")
      end
      i,v = next(options,i)
   end
   write("}\n")
   writeto()
end

function Ask(question)
   write(question.question)
   write("\n")
   write("["..question.default.."] ")

   local option = read()
   if option == "" then option = question.default end
   question.handleAnswer(option)
end

function Choose(question)
   if question.title then
      print(question.title)
   end

   local i = 1
   local current = ""
   while question.options[i] do
      print("   " .. i .. " - " .. question.options[i])
      if question.options[i] == question.default then
         current = i
      end
      i = i + 1
   end
   print("   X - Exit")
   write("["..current.."] ")

   local option = read()
   if option == "" then option = current end
   if option == "x" or option == "X" then exit() end
   option = tonumber(option)
   if not option or option < 1 or option > (i-1) then
      print("** Invalid option **")
      Choose(question)
   else
      question.handleAnswer(option, question.options[option])
   end
end

function getItems(table)
   local items = { n = 0}
   foreach(table, function(i,v) %items.n = %items.n + 1; %items[%items.n] = i end)
   return items
end

-----------------------------------------------------------------------------

--
-- LuaOrb's Configuration Functions
--

function chooseDebug(op)
   op = strlower(op)
   if op == "yes" or op == "y" then
      options.debug = "yes"
   else
      options.debug = "no"
   end
end

function chooseJTC(op)
   op = strlower(op)
   if op == "yes" or op == "y" then
      options.use_jtc = "yes"
   else
      options.use_jtc = "no"
   end

   if options.use_jtc == "yes" then
      Ask{ question = "Please enter the JTC installation path: ",
           default = options.jtc_dir,
           handleAnswer = function (op) options.jtc_dir = op end
      }
   end
end

function writeConfig(file)
   readfrom("mak/"..InputFiles[options.orb][options.os][1])
   local source = read(".*")
   readfrom()

   -- Lua dir
   source = gsub(source,"@lua_dir@",options.lua_dir)

   -- ORB dir
   source = gsub(source,"@orb_dir@",options.orb_dir)

   -- have JTC
   if options.orb == "ORBacus C++ 4.*" or options.orb == "ORBacus C++ 3.*" then
      -- use JTC
      if options.use_jtc == "yes" then
         source = gsub(source,"@use_jtc_inc@","USE_JTC_INC")
         source = gsub(source,"@use_jtc_lib@","USE_JTC_LIB")
      else
         source = gsub(source,"@use_jtc_inc@","#USE_JTC_INC")
         source = gsub(source,"@use_jtc_lib@","#USE_JTC_LIB")
      end
      -- JTC dir
      source = gsub(source,"@jtc_dir@",options.jtc_dir)
   end

   -- debug
   if options.debug == "yes" then
      source = gsub(source,"@use_debug@","USE_DEBUG")
   else
      source = gsub(source,"@use_debug@","#USE_DEBUG")
   end

   writeto(file)
   write(source)
   writeto()
end

function writeMakefile(file)
   readfrom("mak/"..InputFiles[options.orb][options.os][2])
   local source = read(".*")
   readfrom()
   writeto(file)
   write(source)
   writeto()
end

-------------------------------------------------------------------------------------------

--
-- Configuration Script
--


InputFiles = {
   ["ORBacus C++ 4.*"] = {
      ["Linux"] = {"cfglinux-ob4.mak", "ob4.mak"},
      ["Windows NT/95/98"] = {"cfgwin-ob4.mak", "ob4win.mak"},
   },

   ["ORBacus C++ 3.*"] = {
      ["Linux"] = {"cfglinux-ob3.mak", "ob3.mak"},
      ["SunOS 5.5.*"] = {"cfglinux-ob3.mak", "ob3.mak"},
      ["Windows NT/95/98"] = {"cfgwin-ob3.mak", "ob3win.mak"},
   },

   ["Mico 2.3.*"] = {
      ["Linux"] = {"cfglinux-mico.mak", "mico.mak"},
   },

   ["Visibroker C++ 3.3"] = {
      ["Windows NT/95/98"] = {"cfgwin-vb.mak", "vbwin.mak"},
   },
}

options = {
   orb = "ORBacus C++ 4.*",
   os = "Linux",
   orb_dir = "",
   jtc_dir = "",
   use_jtc = "no",
   lua_dir = "",
   debug = "yes",
}

CACHE_FILE = "mak/cache.lua"

dofile(CACHE_FILE)

Choose{
   title = "Choose the ORB:",
   options = getItems(InputFiles),
   default = options.orb,
   handleAnswer = function (op, op_str) options.orb = op_str end
}

Choose{
   title = "Choose the Operating System:",
   options = getItems(InputFiles[options.orb]),
   default = options.os,
   handleAnswer = function (op, op_str) options.os = op_str end
}

Ask{
   question = "Please enter the ORB installation path: ",
   default = options.orb_dir,
   handleAnswer = function (op) options.orb_dir = op end
}

if options.orb == "ORBacus C++ 4.*" or options.orb == "ORBacus C++ 3.*" then
   Ask{
      question = "Did you compile "..options.orb.." with JTC support? ",
      default = options.use_jtc,
      handleAnswer = chooseJTC
   }
end

Ask{
   question = "Please enter the Lua installation path: ",
   default = options.lua_dir,
   handleAnswer = function (op) options.lua_dir = op end
}

Ask{
   question = "Add debug information to the generated code?",
   default = options.debug,
   handleAnswer = chooseDebug
}

saveCache()


writeConfig("config.mak")
writeMakefile("Makefile")

