#include <stdio.h>
#include <iostream>
#include <unistd.h>
#include <termio.h>
#include <sys/select.h>
#include <sys/time.h>
#include <sys/ioctl.h>

using namespace std;

extern "C" {
#include <lua.h>
#include <lualib.h>
}

#include <CORBA.h>
#include <luaorb.h>

#include "utils.h"

int main( int argc, char *argv[] )
{
   struct termio ctl;
   ioctl(STDIN_FILENO, TCGETA, &ctl);
   ctl.c_lflag &= ~(ICANON | ECHO);
   ioctl(STDIN_FILENO, TCSETA, &ctl);

   iolib_open();
   strlib_open();
   mathlib_open();

   // ORB initialization
   CORBA::ORB_var orb = CORBA::ORB_init(argc, argv);
   
   CORBA::Object_var obj = orb -> resolve_initial_references("RootPOA");
   PortableServer::POA_var poa = PortableServer::POA::_narrow(obj);

   luaorb_open(orb, poa);

   int interactive = proc_params(argc,argv);

   if (!interactive)
     orb->run();
   else
   {
      char input[500];
      int  len = 0;
      cout << "LuaOrb Mico Console v. 1.0" << endl;
      cout << "> " << flush;
   
      while (1)
      {
         struct timeval timeout;
         fd_set fdset;
         timeout.tv_sec = 0;
         timeout.tv_usec = 10000;  
         FD_SET(STDIN_FILENO, &fdset);
         if (select(1, &fdset, 0, 0, &timeout)>0)
         {
            char c = getchar();
            switch(c)
            {
            case '\b':
            case 127:
            {
               if(len > 0)
               {
                  input[len - 1] = '\0';
                  len--;
                  cout << "\b \b" << flush;
               }
               break;
            }
            case '\r':
            case '\n':
            {
               cout << endl << flush;
               lua_dostring(input);
               input[0] = '\0';
               len = 0;
               cout << "> " <<  flush;
               break;
            }
            default:
            {
               if(isprint(c))
               {
                  cout << c << flush;
                  input[len] = c;
                  len++;
                  input[len] = '\0';
               }
               break;
            }
            }
         }
         if (orb->work_pending())
            orb->perform_work();
      }
   }

   luaorb_close();
   lua_close();
 
   CORBA::release(poa);
   orb->destroy();
   
   return 0;
}
