#include <stdio.h>

extern "C"
{
#include <lua.h>
#include <lualib.h>
}

#include <OB/CORBA.h>
#include <OB/Properties.h>
#include <OB/Logger.h>
#include <luaorb.h>

#include "wreactor-ob4.h"
#include "utils.h"

CORBA::ORB_ptr orb = NULL;
PortableServer::POA_ptr poa = NULL;


static void my_exit()
{
   orb->shutdown(0);
   luaorb_close();
   WConsoleReactor::Close();
}

int main(int argc, char** argv)
{
   iolib_open();
   strlib_open();
   mathlib_open();

   OB::Reactor_var reactor = WConsoleReactor::Init("LuaOrb Console - version 1.5");

   OB::Properties_var props = OB::Properties::getDefaultProperties();
   OB::ParseArgs(argc, argv, props);
   props->setProperty("ooc.orb.conc_model", "reactive");
   props->setProperty("ooc.orb.oa.conc_model", "reactive");

   orb = OBCORBA::ORB_init(argc, argv, OB::Properties::_nil(),OB::Logger::_nil(),reactor);
   
   CORBA::Object_var obj = orb -> resolve_initial_references("RootPOA");
   PortableServer::POA_ptr poa = PortableServer::POA::_narrow(obj);
      
   OBCORBA::ORB_var obOrb = OBCORBA::ORB::_narrow(orb);

   #if OB_INTEGER_VERSION < 4000100L
   obOrb->run_init();
   #endif

   poa->the_POAManager()->activate();

   luaorb_open(orb,poa);
   
   int interactive = proc_params(argc,argv);


   if (interactive)
      WConsoleReactor::Start();

   lua_register("new_exit",my_exit);
   lua_dostring("original_exit = exit; exit = new_exit");

   orb->run();

   lua_close();

   CORBA::release(poa);
   orb->destroy();
   
   return 0;
}
