#include <stdio.h>

#ifdef ORB_ORBACUS
#include <OB/CORBA.h>
#include <OB/Util.h>
#include <ctype.h>
#include <OB/Timer.h>
#else
#include <dispatch/dispatch.h>
#endif

extern "C"
{
#include <lua.h>
}

#include "cotimer.h"

static int TIMER_TAG = 0;

class CoTimer : 
#ifdef ORB_ORBACUS
  public OBTimer
#else
  public IOHandler
#endif
{
   int lref;
   unsigned int t;
   int locked;

   void lock_ref()
   {
      if (! this->locked)
      {
         lua_Object l_obj = lua_getref(this->lref);
         lua_pushobject(l_obj);
         this->lref = lua_ref(1);
         this->locked = 1;
      }
   }

   void unlock_ref()
   {
      if (this->locked)
      {
         int aux_ref = this->lref;
         lua_Object l_obj = lua_getref(aux_ref);
         lua_pushobject(l_obj);
         this->lref = lua_ref(0);
         this->locked = 0;
         lua_unref(aux_ref);
      }
   }

public:
   CoTimer( lua_Object l_obj, unsigned long _t)
   {
      lua_pushobject(l_obj);
      this->lref = lua_ref(0);
      this->t = _t;
      this->locked = 0;
      //Start();
   }

   #ifndef ORB_ORBACUS
   virtual void timerExpired(long,long)
   {
      this->notify();
   }
   #endif

   virtual void notify()
   {
      lua_beginblock();
      lua_Object l_obj = lua_getref(this->lref);
      lua_pushobject(l_obj);
      lua_pushstring("notify");
      lua_Object l_func = lua_gettable();
      if (lua_isfunction(l_func))
      {
         lua_pushobject(l_obj);
         lua_callfunction(l_func);
         lua_Object result = lua_getresult(1);
         if (result != LUA_NOOBJECT && ! lua_isnil(result))
         {
            this->Start();
         }
         else
         {
            this->unlock_ref();
         }
      }
      lua_endblock();
   }

   void Start()
   {
      this->lock_ref();

      #ifdef ORB_ORBACUS
      struct timeval tv;
      tv.tv_sec =  t/1000000;
      tv.tv_usec = t%1000000;

      activate(tv,true);
      #else
      Dispatcher::instance().startTimer(t/1000000,t%1000000,this);
      #endif
   }

   virtual void Stop()
   {
      this->unlock_ref();
      #ifdef ORB_ORBACUS
      this->stop();
      #else
      Dispatcher::instance().stopTimer(this);
      #endif
   }

   void setInterval(unsigned int new_t) { this->t = new_t; }
};


static void _RegisterTimer()
{
   lua_Object l_obj = lua_getparam(1);
   unsigned long t = (unsigned long) lua_getnumber(lua_getparam(2));
   if (lua_istable(l_obj))
   {
      CoTimer* timer = new CoTimer(l_obj,t);
      lua_pushusertag(timer, TIMER_TAG);
   }
   else
   {
      lua_error("The first parameter is not a table.");
      lua_pushnil();
   }
}

static void _TimerStop()
{
   lua_Object obj = lua_getparam(1);
   if (lua_tag(obj) == TIMER_TAG)
   {
      CoTimer* timer = (CoTimer*) lua_getuserdata(obj);
      timer->Stop();
   }
}

static void _TimerStart()
{
   lua_Object obj = lua_getparam(1);
   if (lua_tag(obj) == TIMER_TAG)
   {
      CoTimer* timer = (CoTimer*) lua_getuserdata(obj);
      timer->Start();
   }
}

static void _TimerInterval()
{
   lua_Object obj = lua_getparam(1);
   unsigned long t = (unsigned long) lua_getnumber(lua_getparam(2));
   if (lua_tag(obj) == TIMER_TAG)
   {
      CoTimer* timer = (CoTimer*) lua_getuserdata(obj);
      if (timer)
      {
         timer->setInterval(t);
      }
   }
}

static void gc_fb()
{
   lua_Object obj = lua_getparam(1);
   if (lua_tag(obj) == TIMER_TAG)
   {
      CoTimer* timer = (CoTimer*) lua_getuserdata(obj);
      if (timer)
      {
         //timer->Stop();
         delete timer;
      }
   }
}

static char timer_code[] =
"function Timer(t)\n"
"   t._ud = RegisterTimer(t,t.interval)\n"
"   t.Start = function (self) TimerStart(self._ud) end\n"
"   t.Stop = function (self) TimerStop(self._ud) end\n"
"   t.setInterval = function (self,t) TimerInterval(self._ud,t) end\n"
"   return t\n"
"end";

void cotimer_open()
{
   TIMER_TAG = lua_newtag();

   lua_pushcfunction(gc_fb);
   lua_settagmethod(TIMER_TAG,"gc");

   lua_register("RegisterTimer",_RegisterTimer);
   lua_register("TimerStart",_TimerStart);
   lua_register("TimerStop",_TimerStop);
   lua_register("TimerInterval",_TimerInterval);

   lua_dostring(timer_code);
}

