#include <OB/CORBA.h>
#include <OB/Util.h>
#include <OB/Reactor.h>
#include <ctype.h>
#include <stdio.h>
#include <signal.h>
#include <unistd.h>

extern "C"{
#include <lua.h>
};

#ifdef HAVE_TERMIO_H
#   include <termio.h>
#else
#   ifndef HAVE_TERMIOS_H
#       error Neither termio.h nor termios.h is available!
#   endif
#   include <termios.h>
#endif
#include <sys/ioctl.h>


#include "kreactor-ob3.h"

#ifdef HAVE_TERMIO_H
static struct termio TerminalSettings;
#else
static struct termios TerminalSettings;
#endif

static void ResetTerm(int sig)
{
    //
    // Reset terminal
    //
#ifdef HAVE_TERMIO_H
    ioctl(STDIN_FILENO, TCSETA, &TerminalSettings);
#else
    ioctl(STDIN_FILENO, TCSETS, &TerminalSettings);
#endif

    //
    // Send signal again
    //
    signal(sig, SIG_DFL);
    kill(getpid(), sig);
}


KReactor::KReactor() : input_(CORBA_string_dup(""))
{
    //
    // Register with the reactor
    //
    OBReactor* reactor = OBReactor::instance();
    reactor -> registerHandler(this, OBEventRead, 0);
    fflush(stdin);

    //setvbuf(stdin,NULL,_IONBF,0);

    //
    // Set terminal mode
    //
#ifdef HAVE_TERMIO_H
    struct termio ctl;
    ioctl(STDIN_FILENO, TCGETA, &TerminalSettings);
    ioctl(STDIN_FILENO, TCGETA, &ctl);
#else
    struct termios ctl;
    ioctl(STDIN_FILENO, TCGETS, &TerminalSettings);
    ioctl(STDIN_FILENO, TCGETS, &ctl);
#endif
    ctl.c_lflag &= ~(ICANON | ECHO);
    ctl.c_cc[VMIN] = 1;
    ctl.c_cc[VTIME] = 0;
#ifdef HAVE_TERMIO_H
    ioctl(STDIN_FILENO, TCSETA, &ctl);
#else
    ioctl(STDIN_FILENO, TCSETS, &ctl);
#endif
    //
    // Set signal handler
    //
    signal(SIGHUP, ResetTerm);
    signal(SIGINT, ResetTerm);
    signal(SIGTERM, ResetTerm);
    signal(SIGQUIT, ResetTerm);
}

KReactor::~KReactor()
{
    //
    // Unregister with the reactor
    //
    OBReactor* reactor = OBReactor::instance();
    reactor -> unregisterHandler(this);

    //
    // Reset terminal
    //
#ifdef HAVE_TERMIO_H
    ioctl(STDIN_FILENO, TCSETA, &TerminalSettings);
#else
    ioctl(STDIN_FILENO, TCSETS, &TerminalSettings);
#endif

}

// ----------------------------------------------------------------------
// ORBacus specific event handling
// ----------------------------------------------------------------------
 
void KReactor::handleStop()
{
    // Do nothing here
}

void KReactor::handleEvent(CORBA_ULong mask)
{
    assert(mask == OBEventRead);

    char c;
    cin.read(&c, 1);

    switch(c)
    {
    case '\b':
    {
        if(strlen(input_) > 0)
        {
            input_[strlen(input_) - 1] = '\0';
            cout << "\b \b" << flush;
        }
        break;
    }
    case '\n':
    {
        cout << c << flush;
        CORBA_String_var s = input_;
        input_[0] = '\0';
        lua_dostring(s);
        cout << "> " <<  flush;
        break;
    }
    default:
    {
        if(isprint(c))
        {
            cout << c << flush;
            input_ += c;
        }  
        break;
    }
    }
}

