#ifndef WREACTOR_H
#define WREACTOR_H

#include <Windows.h>
#include <OB/Reactor.h>
#include <OB/Util.h>

class WConsoleReactor : public OBReactor
{
    //
    // Hide copy-constructor and asignment operator
    //
    WConsoleReactor(const WConsoleReactor&);
    void operator=(const WConsoleReactor&);

    //
    // If set to true, event dispatching must be stopped
    //
    bool stop_;

    //
    // The window used for message notifications
    //
    static HWND hWnd_;
    static unsigned long thread_handle_;

    //
    // Struct for an event handler info
    //
    struct HandlerInfo
    {
       OBEventHandler* handler;
       OBMask mask;
       OBHandle handle;
       HandlerInfo* next;
    };

    //
    // Head of the list with all event handler infos
    //
    HandlerInfo* handlerInfoList_;

    WConsoleReactor(HINSTANCE);
    virtual ~WConsoleReactor();

public:

    //
    // Get the reactor singleton
    //
    static OBReactor* instance(HINSTANCE);

    static void Init(const char* title);
    static void Start();
    static void Close();

    //
    // Register and unregister event handler
    //
    virtual void registerHandler(OBEventHandler*, OBMask, OBHandle);
    virtual void unregisterHandler(OBEventHandler*);

    //
    // Dispatch events
    //
    virtual void dispatch();
    virtual bool dispatchOneEvent(CORBA_Long);

    //
    // Stop event dispatching
    //
    virtual void stop() { stop_ = true; }

    //
    // Windows message processing function
    //
    LRESULT wndProc(HWND, UINT, WPARAM, LPARAM);
};

//
// The windows message viewer class
//
class WConsoleViewer : public OBMessageViewer
{
    //
    // Hide copy-constructor and asignment operator
    //
    WConsoleViewer(const WConsoleViewer&);
    void operator=(const WConsoleViewer&);

protected:

    WConsoleViewer() { }
    virtual ~WConsoleViewer() { }

public:

    //
    // Get the message viewer singleton
    //
    static OBMessageViewer* instance();

    //
    // Print messages
    //
    virtual void message(const char*);
};

#endif
