//*****************************************************************************
//
// LuaOrb Library
// Computer Science Department of PUC-Rio
// Rio de Janeiro, Brazil
//
// $Id: lo_cfg.cpp,v 1.10 2000/09/28 00:26:40 rcerq Exp $
//
//*****************************************************************************


extern "C"
{
#include <lua.h>
}

#include "orb_inc.h"
#include "lo_cfg.h"
#include "lo_proxy.h"

CORBA::ORB_ptr LuaOrbCfg::orb = NULL;
CORBA::Repository_ptr LuaOrbCfg::orb_ir = NULL;

#ifdef CORBA_2_3
PortableServer::POA_ptr LuaOrbCfg::poa = NULL;
DynamicAny::DynAnyFactory_ptr LuaOrbCfg::orb_dynany_factory = NULL;
#else
CORBA::BOA_ptr LuaOrbCfg::boa = NULL;
#endif

int LuaOrbCfg::verbose = 0;
int LuaOrbCfg::enum_numeric_mode = 0;
int LuaOrbCfg::servant_optimization = 1;
int LuaOrbCfg::CFGTAG = 0;

/***************************************************************************/
/*                                                                         */
/*  Funcoes relacionadas ao objeto de configuracao do LuaOrb (LuaOrb_cfg)  */
/*                                                                         */
/****************************************************************************/
static void cfg_enum_as_numbers( lua_Object table, char* attr, lua_Object val)
{
   if (val == LUA_NOOBJECT || lua_isnil(val))
      LuaOrbCfg::enum_numeric_mode = 0;
   else
      LuaOrbCfg::enum_numeric_mode = 1;
}

static void cfg_irep( lua_Object table, char* attr, lua_Object val)
{
   LuaOrb* proxy = NULL;

   if (val != LUA_NOOBJECT)
      proxy  = LuaOrb::from_lua(val);

   if (LuaOrbCfg::orb_ir)
   {
      CORBA_release(LuaOrbCfg::orb_ir);
      LuaOrbCfg::orb_ir = CORBA::Repository::_nil();
   }

   if (val != LUA_NOOBJECT && proxy)
   {
      LuaOrbCfg::orb_ir = CORBA::Repository::_narrow( proxy->orb_object );
   }
}

static void cfg_verbose_mode( lua_Object table, char* attr, lua_Object val)
{
   if (val == LUA_NOOBJECT || lua_isnil(val))
      LuaOrbCfg::verbose = 0;
   else
      LuaOrbCfg::verbose = 1;
}

static void cfg_servant_optimization( lua_Object table, char* attr, lua_Object val)
{
   if (val == LUA_NOOBJECT || lua_isnil(val))
      LuaOrbCfg::servant_optimization = 0;
   else
      LuaOrbCfg::servant_optimization = 1;
}

typedef void (*cfg_fn) (lua_Object, char*, lua_Object);
struct cfg_table
{
   char* attr;
   cfg_fn function;
};

cfg_table cfg_tb []= {{"enum_as_numbers",cfg_enum_as_numbers},
                      {"irep",cfg_irep},
                      {"verbose_mode",cfg_verbose_mode},
                      {"servant_optimization",cfg_servant_optimization}};

static void cfg_settable_fb(void)
{
   lua_Object table = lua_getparam(1);
   lua_Object index = lua_getparam(2);
   lua_Object val = lua_getparam(3);

   if (lua_isstring(index))
   {
      char* attr = lua_getstring(index);
      int n = sizeof(cfg_tb)/sizeof(struct cfg_table);
      for (int i = 0; i < n; i++)
         if (strcmp(attr,cfg_tb[i].attr) == 0)
         {
            cfg_tb[i].function(table,attr,val);
            break;
         }
   }

   lua_pushobject(table);
   lua_pushobject(index);
   lua_pushobject(val);
   lua_rawsettable();
}


#ifdef CORBA_2_3
void LuaOrbCfg::open(CORBA::ORB_ptr orb, PortableServer::POA_ptr root_poa)
#else
void LuaOrbCfg::open(CORBA::ORB_ptr orb, CORBA::BOA_ptr boa)
#endif
{
   //Cria a tabela global que guarda a configuracao do LuaOrb
   LuaOrbCfg::CFGTAG = lua_newtag();

   lua_pushcfunction(cfg_settable_fb);
   lua_settagmethod(LuaOrbCfg::CFGTAG,"settable");

   lua_Object cfg_table = lua_createtable();
   lua_pushobject(cfg_table);
   lua_settag(LuaOrbCfg::CFGTAG);

   lua_pushobject(cfg_table);
   lua_setglobal("LuaOrb_cfg");
   lua_pushobject(cfg_table);
   lua_pushstring("servant_optimization");
   lua_pushnumber(1.0);
   lua_rawsettable();

   LuaOrbCfg::orb = orb;

   #ifdef CORBA_2_3
   CORBA::Object_ptr obj = orb->resolve_initial_references("DynAnyFactory");
   if (CORBA::is_nil(obj))
      lua_error("LuaOrb Initialization Error: DynAny Factory could not be created.");
   LuaOrbCfg::orb_dynany_factory = DynamicAny::DynAnyFactory::_narrow(obj);
   if (CORBA::is_nil(LuaOrbCfg::orb_dynany_factory))
      lua_error("LuaOrb Initialization Error: DynAny Factory could not be created.");

   CORBA::PolicyList policy_list;

   LuaOrbCfg::poa = root_poa->create_POA("LuaOrbPOA",
                                         PortableServer::POAManager::_nil(),
					 policy_list);
   LuaOrbCfg::poa->the_POAManager()->activate();

   #else
   LuaOrbCfg::boa = boa;
   #endif
}

void LuaOrbCfg::close()
{
   if (LuaOrbCfg::orb_ir)
   {
      CORBA_release(LuaOrbCfg::orb_ir);
      LuaOrbCfg::orb_ir = CORBA::Repository::_nil();
      #ifdef CORBA_2_3
      CORBA_release(LuaOrbCfg::orb_dynany_factory);
      LuaOrbCfg::orb_dynany_factory = DynamicAny::DynAnyFactory::_nil();
      #endif
   }

}
