//*****************************************************************************
//
// LuaOrb Library
// Computer Science Department of PUC-Rio
// Rio de Janeiro, Brazil
//
// $Id: lo_class.cpp,v 1.11 2000/10/23 23:49:13 rcerq Exp $
//
//*****************************************************************************


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <orb_inc.h>

extern "C"
{
#include <lua.h>
}

#include <luaorb.h>
#include "lo_class.h"
#include "lo_cfg.h"
#include "lo_except.h"
#include "lua2tk.h"

//**************************** Class LuaOrbClass ****************************

LuaOrbClass* LuaOrbClass::classes = NULL;

void LuaOrbClass::close()
{
   LuaOrbClass* current;
   while ((current = LuaOrbClass::classes))
   {
       LuaOrbClass::classes = current->next;
       delete current;
   }
}

LuaOrbClass* LuaOrbClass::get_class_by_obj(CORBA::Object_ptr obj)
{
   LuaOrbClass* cl = NULL;
   try
   {
      CORBA::InterfaceDef_ptr idef = obj->_get_interface();
      if(CORBA::is_nil(idef))
      {
          throw Err_General("LuaOrb: No Interface Repository Available");
      }
      else
      {
         CORBA::InterfaceDef::FullInterfaceDescription* desc = idef->describe_interface();

         cl = LuaOrbClass::get_class_by_id(desc->id);
         if (cl->_interface_name == NULL)
            cl->interface_name(idef->absolute_name());
         if (cl->_interfacedef == NULL)
            cl->_interfacedef = CORBA::InterfaceDef::_duplicate(idef);
      }
   }
   catch(CORBA::SystemException& ex)
   {
      cout << "Error: method CORBA::Object::_get_interface failed." << endl;
      cout << "Exception: " << ex << endl;
      throw;
   }
   catch(...)
   {
      cout << "Error: method CORBA::Object::_get_interface failed." << endl;
      throw;
   }

   return cl;
}

LuaOrbClass* LuaOrbClass::get_class_by_id(const char* id)
{
   LuaOrbClass* current = LuaOrbClass::classes;

   while (current)
   {
      if (current->_repository_id &&
          strcmp(id, current->_repository_id) == 0)
         return current;
      current = current->next;
   }
   if (current == NULL)
   {
      current = new LuaOrbClass;
      current->repository_id((char*)id);
      current->next = LuaOrbClass::classes;
      LuaOrbClass::classes = current;
   }
   return current;
}

LuaOrbClass* LuaOrbClass::get_class_by_name(const char* iname)
{
   LuaOrbClass* current = LuaOrbClass::classes;

   while (current)
   {
      if (current->_interface_name &&
          strcmp(iname, current->_interface_name) == 0)
         return current;
      current = current->next;
   }
   if (current == NULL)
   {
      current = new LuaOrbClass;
      current->interface_name((char*)iname);
      current->next = LuaOrbClass::classes;
      LuaOrbClass::classes = current;
   }
   return current;
}

LuaOrbClass* LuaOrbClass::get_class_by_interfacedef(CORBA::InterfaceDef_ptr idef)
{
   LuaOrbClass* lo_class = LuaOrbClass::get_class_by_name(idef->name());

   if (CORBA::is_nil(lo_class->_interfacedef))
      lo_class->_interfacedef = CORBA::InterfaceDef::_duplicate(idef);

   return lo_class;
}


CORBA::InterfaceDef_ptr LuaOrbClass::get_interfacedef()
{
   if (CORBA::is_nil(this->_interfacedef))
   {
      CORBA::Contained_ptr contained = NULL;

      //The Interface Repository is required here
      if (CORBA::is_nil(LuaOrbCfg::orb_ir))
         throw Err_General("LuaOrb: No Interface Repository Available");

      if (this->repository_id())
         contained = LuaOrbCfg::orb_ir->lookup_id(this->repository_id());
      else if (this->interface_name())
         contained = LuaOrbCfg::orb_ir->lookup(this->interface_name());

      if (contained)
      {
         CORBA::InterfaceDef_ptr idef = CORBA::InterfaceDef::_narrow(contained);
         if (idef)
            this->_interfacedef = idef;
         else
         {
            char msg[100];
            sprintf(msg,"The Interface %s does not exist",this->interface_name());
            throw Err_General(msg);;
         }
         CORBA_release(contained);
      }
      else
      {
         char msg[100];
         sprintf(msg,"The Interface %s does not exist",this->interface_name());
         throw Err_General(msg);;
      }
   }
   return this->_interfacedef;
}

MethodDescr* LuaOrbClass::get_opdef(char* name)
{
   MethodDescr* descr = this->method_cache->get(name);

   if (!descr)
   {
      //the cache does not have this method description
      CORBA::ContainedSeq* seq = this->get_interfacedef()->lookup_name(name,-1,CORBA(dk_Operation),0);
      if (seq->length() == 1)
      {
         CORBA::OperationDef_ptr opdef=CORBA::OperationDef::_narrow((*seq)[0]);
         if (opdef)
         {
            descr = this->add_opdef(name,opdef);
            CORBA_release(opdef);
         }
      }
      CORBA_release(seq);
   }
   return descr;
}

MethodDescr* LuaOrbClass::add_opdef(char* name, CORBA::OperationDef_ptr def)
{
   MethodDescr* descr = new MethodDescr;
   descr->name = CORBA::string_dup(name);

   CORBA::Contained::Description* orb_descr = def->describe();

   #if defined(CORBA_2_3) && defined(ORB_ORBACUS)
   const CORBA::OperationDescription* aux_ptr;
   #else
   CORBA::OperationDescription* aux_ptr;
   #endif

   orb_descr->value >>= aux_ptr;
   descr->orb_descr = new CORBA::OperationDescription(*aux_ptr);

   this->method_cache->insert(descr);
   delete orb_descr;
   return descr;
}

AttribDescr* LuaOrbClass::get_attdef(char* name)
{
   AttribDescr* descr = this->attr_cache->get(name);

   if (!descr)
   {
      //the cache does not have this attribute description
      CORBA::ContainedSeq* seq = this->get_interfacedef()->lookup_name(name,1,CORBA(dk_Attribute),0);
      if (seq->length() == 1)
      {
         CORBA::AttributeDef_ptr attdef = CORBA::AttributeDef::_narrow((*seq)[0]);
         if (attdef)
         {
            descr = this->add_attdef(name,attdef);
            CORBA_release(attdef);
         }
      }
      CORBA_release(seq);
   }
   return descr;
}

AttribDescr* LuaOrbClass::add_attdef(char* name, CORBA::AttributeDef_ptr def)
{
   AttribDescr* descr = new AttribDescr;
   descr->name = CORBA::string_dup(name);

   CORBA::Contained::Description* orb_descr = def->describe();

   #if defined(CORBA_2_3) && defined(ORB_ORBACUS)
   const CORBA::AttributeDescription* aux_ptr;
   #else
   CORBA::AttributeDescription* aux_ptr;
   #endif

   orb_descr->value >>= aux_ptr;
   descr->orb_descr = new CORBA::AttributeDescription(*aux_ptr);

   this->attr_cache->insert(descr);
   delete orb_descr;
   return descr;
}

//**************************** Class MethodDescr *********************************
void MethodDescr::marshal_params(CORBA::NVList_ptr arguments, LuaObjList* list)
{
    unsigned int i, in_params = 0;
    CORBA::Any anyValue;

    for (i=0;i<this->orb_descr->parameters.length();i++)
    {
       CORBA::Flags mode;
       CORBA::ParameterDescription& param = this->orb_descr->parameters[i];
       if ( param.mode == CORBA(PARAM_IN))
           mode = CORBA(ARG_IN);
       else if ( param.mode == CORBA(PARAM_OUT))
           mode = CORBA(ARG_OUT);
       else if ( param.mode == CORBA(PARAM_INOUT))
           mode = CORBA(ARG_INOUT);
       try
       {
        if ( mode == CORBA(ARG_IN) || mode == CORBA(ARG_INOUT))
        {
           if (in_params >= list->size())
              throw Err_NParams(this->orb_descr->parameters.length(),
                                list->size());

           lua2tc(param.type, list->get(in_params), &anyValue);
           arguments->add_value(param.name,anyValue,mode);
           in_params++;
        }
        else
        {
           CORBA::NamedValue_ptr nv = arguments->add_item(param.name,mode);
           nv->value()->replace(param.type,NULL);
        }
       }
       catch (Err_Param& ex)
       {
        ex.index = in_params+1;
        throw;
       }
    }

}

CORBA::Any* MethodDescr::mount_result(CORBA::Request_ptr req)
{
   if(this->orb_descr->result &&
      this->orb_descr->result->kind() != CORBA(tk_void))
   {
      CORBA::Any* resultAny = NULL;
      CORBA::NamedValue* result = req->result();
      resultAny = result->value();

   #ifdef CORBA_2_3
      req->set_return_type(this->orb_descr->result);
   #else
      resultAny->replace(this->orb_descr->result, &result);
   #endif

      return resultAny;
   }
   else
      return NULL;
}


void MethodDescr::unmarshal_result(CORBA::Any* resultAny, CORBA::NVList_ptr arguments)
{
   //maps the result value
   if (resultAny)
      tc2lua(this->orb_descr->result, resultAny);

    //maps the out params
    unsigned int i;

    for (i=0;i<this->orb_descr->parameters.length();i++)
    {
       CORBA::ParameterDescription& param = this->orb_descr->parameters[i];
       if ( param.mode == CORBA(PARAM_OUT) ||
            param.mode == CORBA(PARAM_INOUT))
       {
          try
          {
             tc2lua(param.type, arguments->item(i)->value());
          }
          catch (Err_Param& ex)
          {
             ex.index = i+1;
             throw;
          }
      }
    }
}

CORBA::NVList_ptr MethodDescr::unmarshal_params(CORBA::ServerRequest_ptr request)
{
   CORBA::NVList_ptr arguments;
   LuaOrbCfg::orb->create_list( 0, arguments );
   unsigned long i;

   for( i=0; i < this->orb_descr->parameters.length(); i++ )
   {
      CORBA(Flags) flags;
      CORBA::ParameterDescription& param = this->orb_descr->parameters[i];

      switch( param.mode )
      {
         case CORBA(PARAM_IN):
            flags = CORBA(ARG_IN);
         break;

         case CORBA(PARAM_OUT):
            flags = CORBA(ARG_OUT);
         break;

         case CORBA(PARAM_INOUT):
            flags = CORBA(ARG_INOUT);
         break;
      }
      CORBA::NamedValue_ptr nv = arguments->add_item(param.name,flags);
      nv->value()->replace(param.type,NULL);
   }

   #ifdef CORBA_2_3
   request->arguments( arguments );
   #else
   request->params( arguments );
   #endif

   CORBA::ULong nParams = arguments->count();
   for( i=0; i<nParams; i++ )
   {
      CORBA::ParameterDescription& param = this->orb_descr->parameters[i];
      if (param.mode != CORBA(PARAM_OUT))
      {
         CORBA::NamedValue_ptr nv = arguments->item(i);
         tc2lua( param.type, nv->value() );
      }
   }
   return arguments;
}

void MethodDescr::marshal_result(CORBA::ServerRequest_ptr request,CORBA::NVList_ptr arguments)
{
   CORBA::Any *result = NULL;
   unsigned int i = 0;
   unsigned int j = 1;

   // Primeiro retorno e correspondente ao tipo
   // do retorno da funcao na IDL.
   // Os seguintes correspondem aos parametros out da IDL.
   if(this->orb_descr->result &&
      this->orb_descr->result->kind() != CORBA(tk_void))
   {
      result = new CORBA::Any;
      lua2tc( this->orb_descr->result, lua_getresult(j), result );
      j++;
   }

   for (i=0;i<this->orb_descr->parameters.length();i++)
   {
      CORBA::ParameterDescription& param = this->orb_descr->parameters[i];
      if ( param.mode == CORBA(PARAM_OUT) || param.mode == CORBA(PARAM_INOUT))
      {
         try
         {
            lua2tc(param.type, lua_getresult(j), arguments->item(i)->value());
            j++;
         }
         catch (Err_Param& ex)
         {
            ex.index = i+1;
            throw;
         }
      }
   }

   if (result)
   #ifdef CORBA_2_3
   {
      const CORBA::Any res = *result;
      request->set_result( res );
   }
   #else
      request->result( result );
   #endif
}

//**************************** Class AttribDescr *********************************
void AttribDescr::marshal_params(CORBA::NVList_ptr arguments, lua_Object l_obj)
{
    CORBA::Any anyValue;
    long mode = CORBA(ARG_IN);

    lua2tc(CORBA::TypeCode::_duplicate(this->orb_descr->type), l_obj, &anyValue);
    arguments->add_value(this->name,anyValue,mode);
}

CORBA::Any* AttribDescr::mount_result(CORBA::Request_ptr req)
{
   if(this->orb_descr->type)
   {
      CORBA::NamedValue* result = req->result();
      CORBA::Any*  resultAny = result->value();

   #ifdef CORBA_2_3
      req->set_return_type(this->orb_descr->type);
   #else
      resultAny->replace(CORBA::TypeCode::_duplicate(this->orb_descr->type), &result);
   #endif

      return resultAny;
   }
   else
      return NULL;
}


void AttribDescr::unmarshal_result(CORBA::Any* resultAny)
{
   if (resultAny)
      tc2lua(CORBA::TypeCode::_duplicate(this->orb_descr->type), resultAny);
}

void AttribDescr::unmarshal_params(CORBA::ServerRequest_ptr request)
{
   // Como esta alterando o valor de um atributo
   // a lista de parameteros so deve conter um 
   // objeto que deve ser empilhado em Lua.
   CORBA::NVList_ptr nvList;
   LuaOrbCfg::orb->create_list(0,nvList);
   nvList->add(CORBA(ARG_IN))->value()->replace(this->orb_descr->type,0);

   #ifdef CORBA_2_3
   request->arguments(nvList);
   #else
   request->params(nvList);
   #endif

   tc2lua(this->orb_descr->type, nvList->item(0)->value());
}

void AttribDescr::marshal_result(CORBA::ServerRequest_ptr request,lua_Object obj)
{
   CORBA::NVList_ptr arguments;
   LuaOrbCfg::orb->create_list( 0, arguments );

   #ifdef CORBA_2_3
   request->arguments( arguments );
   #else
   request->params( arguments );
   #endif

   CORBA::Any* result = new CORBA::Any;
   lua2tc( this->orb_descr->type, obj, result );

   #ifdef CORBA_2_3
   const CORBA::Any res = *result;
   request->set_result( res );
   #else
   request->result( result );
   #endif
}

