//*****************************************************************************
//
// LuaOrb Library
// Computer Science Department of PUC-Rio
// Rio de Janeiro, Brazil
//
// $Id: lo_dsi.cpp,v 1.23 2000/10/18 22:30:59 rcerq Exp $
//
//*****************************************************************************


/*
 * DSIluaman / DSI Lua Manager
 *
 */
#include <assert.h>
#include <stdio.h>
extern "C"
{
#include <lua.h>
#include <lauxlib.h>
}

#include "orb_inc.h"
#include "lo_dsi.h"
#include "lo_proxy.h"
#include "lo_cfg.h"
#include "lua2tk.h"
#include "lo_except.h"
#include "luawrap.h"

#ifdef ORB_ORBACUS
OB_MAKE_NARROW_IMPL_1( LuaDSI, CORBA::DynamicImplementation )
#endif


/***************************************************************************/
/*                                                                         */
/*  Definicoes referentes ao binding com Lua da classe LuaDSI              */
/*                                                                         */
/***************************************************************************/

/*******************************************************************/
/*               Prototipo das funcoes registradas em Lua          */
/*******************************************************************/

static void _CreateDSIServer();

static void index_fb();
static void settable_fb();
static void gc_fb();

static void _get_ior();
static void _get_interface_name();
static void _get_interface();
static void _get_corba_ref();
static void _get_implementation();
static void _set_implementation();


/*******************************************************************/
/*     Variaveis necessarias para o uso do pacote LuaWrapper       */
/*******************************************************************/

LuaDSI* LuaDSI::firstServer = NULL;
LuaWrapper* LuaDSI::Wrapper = NULL;
static char* LuaDSI_TAG = "LuaDSI";

static struct luaL_reg functions_tb []= {{"CreateDSIServer",_CreateDSIServer}};

static struct luaL_reg tagmethods_tb[]= {{"index", index_fb},
                                  {"settable",settable_fb}};

static struct luaL_reg ud_tagmethods_tb[]= {{"gc", gc_fb}};

static struct luaL_reg methods_tb[]= {{"_get_ior", _get_ior},
                               {"_get_interface_name",_get_interface_name},
                               {"_get_interface",_get_interface},
                               {"_get_corba_ref",_get_corba_ref},
                               {"_get_implementation",_get_implementation},
                               {"_set_implementation",_set_implementation}};



/***************************************************************************/
/*                                                                         */
/*  Classe LuaDSI                                                          */
/*                                                                         */
/***************************************************************************/

int LuaDSI::EXCEPTION_TAG = 0;

LuaDSI* LuaDSI::Create(char* InterfaceName, lua_Object lua_obj, char* name)
{
   LuaOrbClass* lo_class = LuaOrbClass::get_class_by_name(InterfaceName);

   if (CORBA::is_nil(lo_class->get_interfacedef()))
   {
      char msg[100];
      sprintf(msg,"The Interface %s does not exist",InterfaceName);
      throw Err_General(msg);;
   }

   // Cria o objeto DSI associado a tabela Lua
   LuaDSI* server =  new LuaDSI(lua_obj, lo_class, name);
   //LuaDSI::_duplicate( server );
   return server;
}

LuaDSI* LuaDSI::Create(char* InterfaceName, char* id, lua_Object lua_obj, char* name)
{
   LuaOrbClass* lo_class = LuaOrbClass::get_class_by_id(id);

   if (CORBA::is_nil(lo_class->get_interfacedef()))
   {
      char msg[100];
      sprintf(msg,"The Interface %s does not exist",id);
      throw Err_General(msg);;
   }

   // Cria o objeto DSI associado a tabela Lua
   lo_class->interface_name(InterfaceName);
   LuaDSI* server =  new LuaDSI(lua_obj, lo_class, name);
   //LuaDSI::_duplicate( server );
   return server;
}

LuaDSI* LuaDSI::Create(CORBA::InterfaceDef_ptr intf, lua_Object lua_obj, char* name)
{
   LuaOrbClass* lo_class = LuaOrbClass::get_class_by_interfacedef(intf);

   // Cria o objeto DSI associado a tabela Lua
   LuaDSI* server =  new LuaDSI(lua_obj, lo_class, name);
   //LuaDSI::_duplicate( server );
   return server;
}

LuaDSI::LuaDSI( lua_Object table, LuaOrbClass* lo_class, char* name)
#ifdef ORB_VISIBROKER
: CORBA_DynamicImplementation((const char*)lo_class->interface_name(),name,(const char*)lo_class->repository_id())
#endif
{
   myclass = lo_class;
   tableLuaRef = -1;

   this->SetLuaObject(table);

   #ifdef CORBA_2_3
   PortableServer::ObjectId_var oid = LuaOrbCfg::poa->activate_object(this);
   #elif defined ORB_ORBACUS
   if (name)
      LuaOrbCfg::orb->connect(this,name);
   else
      LuaOrbCfg::orb->connect(this);
   #elif defined ORB_MICO
   LuaOrbCfg::boa->obj_is_ready(this,0);
   #else
   LuaOrbCfg::boa->obj_is_ready(this);
   #endif

   LuaDSI::insertServer(this);
}

LuaDSI::~LuaDSI()
{
   lua_unref( tableLuaRef );
   if (this->nextServer || this->prevServer)
      LuaDSI::removeServer(this);

   #ifdef CORBA_2_3
//   PortableServer::ObjectId_var oid = LuaOrbCfg::poa->servant_to_id(this);
//   LuaOrbCfg::poa->deactivate_object(oid);
   #else
   LuaOrbCfg::boa->deactivate_obj(this);
   #endif
}

lua_Object LuaDSI::GetLuaObject()
{
   return lua_getref(this->tableLuaRef);
}

void LuaDSI::SetLuaObject( lua_Object table )
{
   if (tableLuaRef != -1)
   {
      lua_unref( tableLuaRef );
      tableLuaRef = -1;
   }

   if( !lua_istable( table ) )
      return;

   lua_pushobject( table );
   tableLuaRef = lua_ref( 1 );

   lua_pushref( tableLuaRef );
   lua_pushstring( "_CORBA_LuaDSI" );
   lua_pushuserdata( (void *)this );
   lua_rawsettable();
}

CORBA::Boolean LuaDSI::_is_a(const char *name)
{
   CORBA::InterfaceDef_ptr intfDef = this->_get_interface();
   CORBA::Boolean result = intfDef->is_a( name );
   return result;
}

CORBA::InterfaceDef_ptr LuaDSI::_get_interface()
{
   return CORBA::InterfaceDef::_duplicate( this->myclass->get_interfacedef() );
}

#ifdef CORBA_2_3
CORBA::RepositoryId LuaDSI::_primary_interface(const PortableServer::ObjectId& oid, PortableServer::POA_ptr poa)
{
   CORBA::InterfaceDef_var idef = this->_get_interface();
   CORBA::InterfaceDef::FullInterfaceDescription_var desc =
       idef -> describe_interface();

   return CORBA::string_dup(desc->name);
}
#endif


//ifdef ORB_MICO
//void LuaDSI::invoke(CORBA::ServerRequest_ptr request, CORBA::Environment&)
void LuaDSI::invoke(CORBA::ServerRequest_ptr request)
{
   try
   {
      #ifdef CORBA_2_3
      const char* op_name = request->operation();
      #else
      const char* op_name = request->op_name();
      #endif

      MethodDescr* op_descr = this->myclass->get_opdef((char*)op_name);
      if (op_descr)
      {
         if (LuaOrbCfg::verbose)
            cout << "Invoke Operation: " << op_name << endl;

         // Obtem a funcao lua responsavel por essa operacao
         lua_beginblock();
         lua_Object value;
         lua_pushref( tableLuaRef );
         lua_pushstring((char*)op_name);
         value = lua_gettable();

         // Armazena na pilha os parametros sendo que o primeiro
         // e o self.
         lua_pushref( tableLuaRef );

         CORBA::NVList_ptr nvList = op_descr->unmarshal_params(request);
         lua_callfunction( value );
         lua_Object ret = lua_getresult(1);
         if (lua_tag(ret) == LuaDSI::EXCEPTION_TAG)
            this->RaiseException(request,op_descr,ret);
         else
            op_descr->marshal_result(request,nvList);

         lua_endblock();
      }
      else
      {
         // Tenta tirar o _get_ e o _set_ e faz nova pesquisa
         char* name = (char*)op_name + 5;
         AttribDescr* attr_descr = this->myclass->get_attdef(name);
         if (attr_descr)
         {
            lua_beginblock();
            lua_pushref( tableLuaRef );
            // +5, pula o _get_ e _set_ dos atributos
            lua_pushstring( name );

            // Verifica se e' um set ou um get do atributo
            if( strncmp( "_set_", op_name, 5 )==0 )
            {
               if (LuaOrbCfg::verbose)
                  cout << "Setting Attribute: " << name << endl;

               attr_descr->unmarshal_params(request);
               lua_settable();
            }
            else
            {
               if (LuaOrbCfg::verbose)
                  cout << "Getting Attribute: " << name << endl;

               lua_Object obj = lua_gettable();
               attr_descr->marshal_result(request,obj);
            }
            lua_endblock();
         }
         else
         {
            if (LuaOrbCfg::verbose)
               cout << "Error calling " << op_name << endl;

            #ifdef CORBA_2_3
            throw CORBA::NO_IMPLEMENT();
            #elif defined ORB_ORBACUS
               CORBA::Any *any = new CORBA::Any;
               *any <<= new CORBA_NO_IMPLEMENT;
               request->exception( any );
            #else
               throw CORBA::NO_IMPLEMENT();
            #endif
         }
      }
   }
   catch( CORBAEXP(BAD_PARAM) )
   {
      cout << "Exception Bad Param" << endl;
   }
   catch( CORBAEXP(MARSHAL) )
   {
      cout << "Exception Marshal" << endl;
   }
   catch( CORBAEXP(BAD_TYPECODE) )
   {
      cout << "Exception bad typecode" << endl;
   }
   catch( CORBAEXP(BAD_OPERATION) )
   {
      cout << "Exception bad operation" << endl;
   }
   catch( CORBA::SystemException& )
   {
      cout << "System Exception ERROR!" << endl;   
   }
   catch (Err_General& excep)
   {
      cout << excep.msg << endl;
   }
   catch( ... )
   {
      cout << "Exception ERROR!" << endl;
   }
}

void LuaDSI::RaiseException(CORBA::ServerRequest_ptr request,MethodDescr* descr,
                            lua_Object exception)
{
   if (descr->orb_descr->exceptions.length() >= 1)
   {
      unsigned int i;
      lua_pushobject(exception);
      lua_pushnumber(1.0);
      char* ex_name = lua_getstring(lua_gettable());
      if (ex_name)
      {
         for (i = 0; i < descr->orb_descr->exceptions.length(); i++)
         {
            if (strcmp(ex_name,descr->orb_descr->exceptions[i].type->name())==0)
            {
               CORBA::Any* anyValue = new CORBA::Any;
               lua2tc(CORBA::TypeCode::_duplicate(descr->orb_descr->exceptions[i].type),
                      exception,anyValue);

               #ifdef CORBA_2_3
               const CORBA::Any any = *anyValue;
               request->set_exception(any);
               #else
               request->exception(anyValue);
               #endif      

               return;
            }
         }
      }
   }
}

void LuaDSI::open()
{
   static char init_code[] = "function LuaOrbException(t)\n"
                                   "   settag(t,LUADSI_EXCEPTION_TAG)\n"
                                   "   return t\n"
                                   "end";
   luaL_openlib(functions_tb,sizeof(functions_tb)/sizeof(struct luaL_reg));

   LuaDSI::Wrapper = LuaWrapper::createWrapper(LuaDSI_TAG);
   LuaDSI::Wrapper->reg_tagmethods(tagmethods_tb,
                          sizeof(tagmethods_tb)/sizeof(struct luaL_reg));
   LuaDSI::Wrapper->reg_methods(methods_tb,
                          sizeof(methods_tb)/sizeof(struct luaL_reg));
   LuaDSI::Wrapper->reg_ud_tagmethods(ud_tagmethods_tb,
                            sizeof(ud_tagmethods_tb)/sizeof(struct luaL_reg));

   LuaDSI::EXCEPTION_TAG = lua_newtag();
   lua_pushnumber(LuaDSI::EXCEPTION_TAG);
   lua_setglobal("LUADSI_EXCEPTION_TAG");
   lua_dostring(init_code);
}

void LuaDSI::close()
{
   if (LuaDSI::Wrapper)
      delete LuaDSI::Wrapper;

   LuaDSI::destroyServerList();
}

void LuaDSI::destroyServerList()
{
   LuaDSI* server;
   while((server=LuaDSI::firstServer))
   {
      LuaDSI::removeServer(server);

      #ifdef CORBA_2_3
      PortableServer::ObjectId_var oid = LuaOrbCfg::poa->servant_to_id(server);
      LuaOrbCfg::poa->deactivate_object(oid);
      server->_remove_ref();
      #else
      LuaOrbCfg::boa->deactivate_obj(server);
      CORBA_release(server);
      #endif

   }
}

void LuaDSI::insertServer(LuaDSI* server)
{
   server->nextServer = LuaDSI::firstServer;
   server->prevServer = 0;
   if (LuaDSI::firstServer)
      LuaDSI::firstServer->prevServer = server;
   LuaDSI::firstServer = server;
}

void LuaDSI::removeServer(LuaDSI* server)
{
   if (server->nextServer)
      server->nextServer->prevServer = server->prevServer;

   if (server->prevServer)
      server->prevServer->nextServer = server->nextServer;

   if (server == LuaDSI::firstServer)
      LuaDSI::firstServer = server->nextServer;
   
   server->nextServer = 0;
   server->prevServer = 0;
}

/***************************************************************************/
/*                                                                         */
/*  Construtor Lua de Objetos LuaDSI                                       */
/*                                                                         */
/****************************************************************************/

static void _CreateDSIServer()
{
   char* interface_name = luaL_check_string(1);
   lua_Object lobj = luaL_tablearg(2);
   lua_Object lname = lua_getparam(3);
   char* name = NULL;

   if (lua_isstring(lname))
      name = lua_getstring(lname);

   try {
      LuaDSI* dserver = LuaDSI::Create(interface_name, lobj, name);

      #ifdef CORBA_2_3
      dserver->_add_ref();
      #else
      LuaDSI::_duplicate( dserver );
      #endif

      LuaDSI::Wrapper->push(dserver);
   }
   catch (Err_General& excep)
   {
      lua_error(excep.msg);
   }
   #ifdef CORBA_2_3
   catch(const CORBA::Exception & excep )
   {
      cout << excep <<endl;
      lua_error("CORBA exception ERROR");
   }
   #endif

}

/***************************************************************************/
/*                                                                         */
/*  Tag-Methods para objetos LuaDSI                                        */
/*                                                                         */
/***************************************************************************/

static void gc_fb(void)
{
   LuaDSI* dserver = (LuaDSI*) lua_getuserdata(lua_getparam(1));

   #ifdef CORBA_2_3
   dserver->_remove_ref();
   #else
   CORBA_release(dserver);
   #endif
}

static void index_fb()
{
   LuaDSI* dserver = (LuaDSI*) LuaDSI::Wrapper->check_tag(1);
   lua_Object index = lua_getparam(2);

   lua_Object lobj = dserver->GetLuaObject();

   lua_pushobject(lobj);
   lua_pushobject(index);
   lua_Object result = lua_gettable();
   lua_pushobject(result);
}

static void settable_fb()
{
   lua_Object table = lua_getparam(1);
   LuaDSI* dserver = (LuaDSI*) LuaDSI::Wrapper->from_lua(table);
   lua_Object index = lua_getparam(2);
   lua_Object val = lua_getparam(3);

   if (lua_isstring(index))
   {
      char* field = lua_getstring(index);
      int i = 0;
      int n = sizeof(methods_tb)/sizeof(struct luaL_reg);
      for (i=0; i < n; i ++)
      {
          if ( strcmp(methods_tb[i].name, field) == 0 )
          {
             lua_pushobject(table);
             lua_pushobject(index);
             lua_pushobject(val);
             lua_rawsettable();
          } 
      }
   }

   lua_pushobject(dserver->GetLuaObject());
   lua_pushobject(index);
   lua_pushobject(val);
   lua_settable();
}

/***************************************************************************/
/*                                                                         */
/*  Operacoes pre-definidas dos servidores dinamicos LuaDSI                */
/*                                                                         */
/****************************************************************************/

static void _get_ior()
{
   LuaDSI* dserver = (LuaDSI*) LuaDSI::Wrapper->check_tag(1);

   #ifdef CORBA_2_3
   CORBA::Object_ptr obj = LuaOrbCfg::poa->servant_to_reference(dserver);
   char* ior = LuaOrbCfg::orb->object_to_string(obj);
   #else
   char* ior = LuaOrbCfg::orb->object_to_string(dserver);
   #endif

   lua_pushstring(ior);
   CORBA::string_free(ior);
}

static void _get_interface_name()
{
   LuaDSI* dserver = (LuaDSI*) LuaDSI::Wrapper->check_tag(1);

   if (dserver->myclass->interface_name())
      lua_pushstring(dserver->myclass->interface_name());
   else
      lua_pushnil();
}

static void _get_interface()
{
   LuaDSI* dserver = (LuaDSI*) LuaDSI::Wrapper->check_tag(1);

   LuaOrb* newproxy = LuaOrb::new_obj(NULL,"CORBA::InterfaceDef",
                                      dserver->myclass->get_interfacedef());
   if (newproxy)
   {
      newproxy->locked = 1;
      LuaOrb::push(newproxy);
   }
   else
      lua_pushnil();
}

static void _get_corba_ref()
{
   LuaDSI* dserver = (LuaDSI*) LuaDSI::Wrapper->check_tag(1);

   lua_pushuserdata(dserver);
}

static void _get_implementation()
{
   LuaDSI* dserver = (LuaDSI*) LuaDSI::Wrapper->check_tag(1);

   lua_pushobject(dserver->GetLuaObject());
}

static void _set_implementation()
{
   LuaDSI* dserver = (LuaDSI*) LuaDSI::Wrapper->check_tag(1);
   lua_Object lobj = luaL_tablearg(2);

   dserver->SetLuaObject(lobj);
}


/////////////////////////////////////////////////////////////////////////
