//*****************************************************************************
//
// LuaOrb Library
// Computer Science Department of PUC-Rio
// Rio de Janeiro, Brazil
//
// $Id: lo_omni.cpp,v 1.13 2000/10/10 12:42:47 rcerq Exp $
//
//*****************************************************************************


#include <orb_inc.h>
#include <lua.h>

#include "lo_cfg.h"
#include "lo_compt.h"


/*************************************************************************/
class Union_Marshal : public Compat::Marshal
{
   DYNANY(DynUnion_var) dyn_union;
   DYNANY(DynAny_var) cur_component;
   int state;
public:
      Union_Marshal(CORBA::TypeCode_ptr tc);
      virtual ~Union_Marshal() { dyn_union->destroy(); }
      void put(CORBA::Any& any);
      void to_any(CORBA::Any* any);
};

Union_Marshal::Union_Marshal(CORBA::TypeCode_ptr tc)
{
   #ifdef CORBA_2_3
   DynamicAny::DynAny_var tmp =
      LuaOrbCfg::orb_dynany_factory->create_dyn_any_from_type_code(tc);
   dyn_union = DynamicAny::DynUnion::_narrow(tmp);
   #else
   dyn_union = LuaOrbCfg::orb->create_dyn_union(tc);
   #endif

   state = 0;
}

void Union_Marshal::put(CORBA::Any& any)
{

   if ( state==0 && any.type()->kind() == CORBA(tk_octet) )
   {
      state = 2;
      #ifdef CORBA_2_3
      dyn_union->set_to_default_member();
      #else
      dyn_union->set_as_default(1);
      #endif
   }
   else if (state == 2)
   {
      DYNANY(DynAny_var) member = dyn_union->member();
      member->from_any(any);
   }
   else
   {
      #ifdef CORBA_2_3
      state = 2;
      DYNANY(DynAny_var) disc = dyn_union->get_discriminator();
      disc->from_any(any);
      #else
      state = 1;
      cur_component = dyn_union->current_component();
      cur_component->from_any(any);
      dyn_union->next();
      #endif
   }
}

void Union_Marshal::to_any(CORBA::Any* any)
{
   *any = *(dyn_union->to_any());
}

/*************************************************************************/
class Struct_Marshal : public Compat::Marshal
{
   DYNANY(DynStruct_var) dyn_struct;
   DYNANY(DynAny_var) cur_component;
public:
      Struct_Marshal(CORBA::TypeCode_ptr tc);
      virtual ~Struct_Marshal() { dyn_struct->destroy(); }
      void put(CORBA::Any& any);
      void to_any(CORBA::Any* any);
};

Struct_Marshal::Struct_Marshal(CORBA::TypeCode_ptr tc)
{
   #ifdef CORBA_2_3
   DynamicAny::DynAny_var tmp =
      LuaOrbCfg::orb_dynany_factory->create_dyn_any_from_type_code(tc);
   dyn_struct = DynamicAny::DynStruct::_narrow(tmp);
   #else
   dyn_struct = LuaOrbCfg::orb->create_dyn_struct(tc);
   #endif

   #if 0
   CORBA::Any any;
   any.replace(tc,NULL);
   DYNANY(DynAny*) dany = LuaOrbCfg::orb->create_dyn_any(any);
   dyn_struct = CORBA::DynStruct::_narrow(dany);
   #endif
}

void Struct_Marshal::put(CORBA::Any& any)
{
   cur_component = dyn_struct->current_component();
   cur_component->from_any(any);
   dyn_struct->next();
}

void Struct_Marshal::to_any(CORBA::Any* any)
{
   *any = *(dyn_struct->to_any());
}

/*************************************************************************/
class Sequence_Marshal : public Compat::Marshal
{
   DYNANY(DynSequence_var) dyn_seq;
   DYNANY(DynAny_var) cur_component;
public:
      Sequence_Marshal(CORBA::TypeCode_ptr tc, unsigned long size);
      virtual ~Sequence_Marshal() { dyn_seq->destroy(); }
      void put(CORBA::Any& any);
      void to_any(CORBA::Any* any);
};

Sequence_Marshal::Sequence_Marshal(CORBA::TypeCode_ptr tc, unsigned long size)
{
   #ifdef CORBA_2_3
   DynamicAny::DynAny_var tmp =
      LuaOrbCfg::orb_dynany_factory->create_dyn_any_from_type_code(tc);
   dyn_seq = DynamicAny::DynSequence::_narrow(tmp);
   dyn_seq->set_length(size);
   #else
   dyn_seq = LuaOrbCfg::orb->create_dyn_sequence(tc);
   dyn_seq->length(size);
   #endif

}

void Sequence_Marshal::put(CORBA::Any& any)
{
   cur_component = dyn_seq->current_component();
   cur_component->from_any(any);
   dyn_seq->next();
}

void Sequence_Marshal::to_any(CORBA::Any* any)
{
   *any = *(dyn_seq->to_any());
}

/*************************************************************************/
class Array_Marshal : public Compat::Marshal
{
   DYNANY(DynArray_var) dyn_array;
   DYNANY(DynAny_var) cur_component;
public:
      Array_Marshal(CORBA::TypeCode_ptr tc);
      virtual ~Array_Marshal() { dyn_array->destroy(); }
      void put(CORBA::Any& any);
      void to_any(CORBA::Any* any);
};

Array_Marshal::Array_Marshal(CORBA::TypeCode_ptr tc)
{
   #ifdef CORBA_2_3
   DynamicAny::DynAny_var tmp =
      LuaOrbCfg::orb_dynany_factory->create_dyn_any_from_type_code(tc);
   dyn_array = DynamicAny::DynArray::_narrow(tmp);
   #else
   dyn_array = LuaOrbCfg::orb->create_dyn_array(tc);
   #endif
}

void Array_Marshal::put(CORBA::Any& any)
{
   cur_component = dyn_array->current_component();
   cur_component->from_any(any);
   dyn_array->next();
}

void Array_Marshal::to_any(CORBA::Any* any)
{
   *any = *(dyn_array->to_any());
}

/*************************************************************************/
class Union_Unmarshal : public Compat::Unmarshal
{
   DYNANY(DynUnion_var) dyn_union;
   DYNANY(DynAny_var) cur_component;
public:
   Union_Unmarshal (CORBA::TypeCode_ptr tc, const CORBA::Any& any);
   virtual ~Union_Unmarshal () { dyn_union->destroy(); }
   CORBA::Any* get();
};

Union_Unmarshal::Union_Unmarshal (CORBA::TypeCode_ptr tc,
                                    const CORBA::Any& any)
{
   #ifdef CORBA_2_3
   DynamicAny::DynAny_var tmp =
      LuaOrbCfg::orb_dynany_factory->create_dyn_any(any);
   dyn_union = DynamicAny::DynUnion::_narrow(tmp);
   #else
   dyn_union = LuaOrbCfg::orb->create_dyn_union(tc);
   dyn_union->from_any(any);
   #endif
}

CORBA::Any* Union_Unmarshal::get()
{
   CORBA::Any* any;
   cur_component = dyn_union->current_component();
   any = cur_component->to_any();
   dyn_union->next();
   return any;
}

/*************************************************************************/
class Struct_Unmarshal : public Compat::Unmarshal
{
   DYNANY(DynStruct_var) dyn_struct;
   DYNANY(DynAny_var) cur_component;
public:
   Struct_Unmarshal (CORBA::TypeCode_ptr tc, const CORBA::Any& any);
   virtual ~Struct_Unmarshal () { dyn_struct->destroy(); }
   CORBA::Any* get();
};

Struct_Unmarshal::Struct_Unmarshal (CORBA::TypeCode_ptr tc,
                                    const CORBA::Any& any)
{
   #ifdef CORBA_2_3
   DynamicAny::DynAny_var tmp =
      LuaOrbCfg::orb_dynany_factory->create_dyn_any(any);
   dyn_struct = DynamicAny::DynStruct::_narrow(tmp);
   #else
   dyn_struct = LuaOrbCfg::orb->create_dyn_struct(tc);
   dyn_struct->from_any(any);
   #endif
}

CORBA::Any* Struct_Unmarshal::get()
{
   CORBA::Any* any;
   cur_component = dyn_struct->current_component();
   any = cur_component->to_any();
   dyn_struct->next();
   return any;
}

/*************************************************************************/
class Sequence_Unmarshal : public Compat::Unmarshal
{
   DYNANY(DynSequence_var) dyn_seq;
   DYNANY(DynAny_var) cur_component;
public:
   Sequence_Unmarshal (CORBA::TypeCode_ptr tc, const CORBA::Any& any);
   virtual ~Sequence_Unmarshal () { dyn_seq->destroy(); }
   CORBA::Any* get();
   unsigned long length() {
      #ifdef CORBA_2_3
      return dyn_seq->get_length();
      #else
      return dyn_seq->length();
      #endif
   }
};

Sequence_Unmarshal::Sequence_Unmarshal (CORBA::TypeCode_ptr tc,
                                    const CORBA::Any& any)
{
   #ifdef CORBA_2_3
   DynamicAny::DynAny_var tmp =
      LuaOrbCfg::orb_dynany_factory->create_dyn_any(any);
   dyn_seq = DynamicAny::DynSequence::_narrow(tmp);
   #else
   dyn_seq = LuaOrbCfg::orb->create_dyn_sequence(tc);
   dyn_seq->from_any(any);
   #endif
}

CORBA::Any* Sequence_Unmarshal::get()
{
   CORBA::Any* any;
   cur_component = dyn_seq->current_component();
   any = cur_component->to_any();
   dyn_seq->next();
   return any;
}

/*************************************************************************/
class Array_Unmarshal : public Compat::Unmarshal
{
   DYNANY(DynArray_var) dyn_array;
   DYNANY(DynAny_var) cur_component;
public:
   Array_Unmarshal (CORBA::TypeCode_ptr tc, const CORBA::Any& any);
   virtual ~Array_Unmarshal () { dyn_array->destroy(); }
   CORBA::Any* get();
};

Array_Unmarshal::Array_Unmarshal (CORBA::TypeCode_ptr tc,
                                    const CORBA::Any& any)
{
   #ifdef CORBA_2_3
   DynamicAny::DynAny_var tmp =
      LuaOrbCfg::orb_dynany_factory->create_dyn_any(any);
   dyn_array = DynamicAny::DynArray::_narrow(tmp);
   #else
   dyn_array = LuaOrbCfg::orb->create_dyn_array(tc);
   dyn_array->from_any(any);
   #endif
}

CORBA::Any* Array_Unmarshal::get()
{
   CORBA::Any* any;
   cur_component = dyn_array->current_component();
   any = cur_component->to_any();
   dyn_array->next();
   return any;
}

#if 0
#ifndef ORB_ORBACUS
/*************************************************************************/
class VB_Exception_Unmarshal : public Compat::Unmarshal
{
   CORBA::MarshalInBuffer buf;
   CORBA::TypeCode_ptr typecode;
   CORBA::Any any_temp;
   unsigned long current;
public:
   VB_Exception_Unmarshal (CORBA::TypeCode_ptr tc, const CORBA::Any& any);
   CORBA::Any* get();
};

VB_Exception_Unmarshal::VB_Exception_Unmarshal (CORBA::TypeCode_ptr tc,
                                    const CORBA::Any& any)
:  buf((char*)any.value(),any.len())
{
   typecode = tc;
   current = 0;
}

CORBA::Any* VB_Exception_Unmarshal::get()
{
   any_temp.read_value(buf,typecode->member_type(current));
   current++;
   return &any_temp;
}

/*************************************************************************/
class VB_Exception_Marshal : public Compat::Marshal
{
protected:
   CORBA::MarshalOutBuffer buf;
   CORBA::TypeCode_ptr typecode;
public:
      VB_Exception_Marshal(CORBA::TypeCode_ptr tc);
      void put(CORBA::Any& any);
      void to_any(CORBA::Any* any);
};

VB_Exception_Marshal::VB_Exception_Marshal(CORBA::TypeCode_ptr tc)
{
   typecode = tc;
}

void VB_Exception_Marshal::put(CORBA::Any& any)
{
   any.write_value(buf);
}

void VB_Exception_Marshal::to_any(CORBA::Any* any)
{
   any->replace(typecode,buf);
}
#endif
#endif

/*************************************************************************/
Compat::Marshal* Compat::create_union_marshaller(CORBA::TypeCode_ptr tc)
{
   return new Union_Marshal(tc);
}

Compat::Marshal* Compat::create_struct_marshaller(CORBA::TypeCode_ptr tc)
{
#if 0
   #ifndef ORB_ORBACUS
   if (tc->kind() == CORBA(tk_except))
      return new VB_Exception_Marshal(tc);
   else
   #endif
#endif
   return new Struct_Marshal(tc);
}

Compat::Marshal* Compat::create_sequence_marshaller(CORBA::TypeCode_ptr tc,
                                        unsigned long size)
{
   return new Sequence_Marshal(tc, size);
}

Compat::Marshal* Compat::create_array_marshaller(CORBA::TypeCode_ptr tc)
{
   return new Array_Marshal(tc);
}

Compat::Unmarshal* Compat::create_union_unmarshaller(CORBA::TypeCode_ptr tc,
                                          const CORBA::Any& any)
{
   return new Union_Unmarshal(tc,any);
}

Compat::Unmarshal* Compat::create_struct_unmarshaller(CORBA::TypeCode_ptr tc,
                                          const CORBA::Any& any)
{
#if 0
   #ifndef ORB_ORBACUS
   if (tc->kind() == CORBA(tk_except))
      return new VB_Exception_Unmarshal(tc,any);
   else
   #endif
#endif
   return new Struct_Unmarshal(tc,any);
}

Compat::Unmarshal* Compat::create_sequence_unmarshaller(
                                            CORBA::TypeCode_ptr tc,
                                            const CORBA::Any& any,
                                            unsigned long* length)
{
   Sequence_Unmarshal* buf = new Sequence_Unmarshal(tc,any);
   *length = buf->length();
   return buf;
}

Compat::Unmarshal* Compat::create_array_unmarshaller(CORBA::TypeCode_ptr tc,
                                         const CORBA::Any& any)
{
   return new Array_Unmarshal(tc,any);
}


