//*****************************************************************************
//
// LuaOrb Library
// Computer Science Department of PUC-Rio
// Rio de Janeiro, Brazil
//
// $Id: lo_proxy.cpp,v 1.48 2000/10/23 23:51:32 rcerq Exp $
//
//*****************************************************************************


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <orb_inc.h>

extern "C"
{
#include <lua.h>
#include <lauxlib.h>
}

#include <luaorb.h>
#include "lua2tk.h"
#include "lo_proxy.h"
#include "lo_cfg.h"
#include "lo_except.h"

static void _get_ior(void);
static void _get_interface_name(void);
static void _get_interface_def(void);
static void _get_repository_id(void);
static void _get_corba_ref(void);


//**************************** Class LuaOrb *********************************

int LuaOrb::TAG = 0;
int LuaOrb::UDTAG = 0;
char* LuaOrb::DEFCLOSURE_TAG = "LO_DEFCLOSURE_TAG";
LuaWrapper* LuaOrb::DefClosureWrapper = NULL;
 

LuaOrb* LuaOrb::new_obj(const char* id, const char* iname,
                        CORBA::Object_ptr orb_obj)
{
   LuaOrb* proxy = new LuaOrb(id,iname,orb_obj);

   return proxy;
}

void LuaOrb::push(LuaOrb* proxy)
{
   lua_pushobject(proxy->GetLuaThis());
}

LuaOrb* LuaOrb::check_tag(int index)
{
   LuaOrb* proxy = LuaOrb::from_lua(lua_getparam(index));

   luaL_arg_check((proxy!=NULL),index,"it isn't a Corba Object");

   return proxy;
}

LuaOrb* LuaOrb::from_lua(lua_Object l_obj)
{
   if (lua_istable(l_obj) && lua_tag(l_obj) == LuaOrb::TAG)
   {
      lua_pushobject(l_obj);
      lua_pushstring("_CORBA_OBJ_REF_");
      return (LuaOrb*) lua_getuserdata(lua_gettable());
   }
   else
      return NULL;
}

LuaOrb::LuaOrb(const char* id, const char* interfacename,
               CORBA::Object_ptr orb_obj)
: orb_object(orb_obj)
{
   this->locked = 0;

   if (id == NULL && interfacename == NULL)
      this->myclass = LuaOrbClass::get_class_by_obj(this->orb_object);
   else if (id)
   {
      this->myclass = LuaOrbClass::get_class_by_id(id);
      if (interfacename && this->myclass->interface_name() == NULL)
         this->myclass->interface_name((char*)interfacename);
   }
   else if (interfacename)
      this->myclass = LuaOrbClass::get_class_by_name(interfacename);
   else
      this->myclass = NULL;

   this->CreateLuaThis();
}

LuaOrb::~LuaOrb()
{
   CORBA_release(this->orb_object);
   lua_unref(this->lua_this);
}

void LuaOrb::CreateLuaThis()
{
   lua_Object table = lua_createtable();

   lua_pushobject(table);
   lua_pushstring("_CORBA_OBJ_REF_");
   lua_pushusertag(this,LuaOrb::UDTAG);
   lua_settable();

   lua_pushobject(table);
   lua_pushstring("_get_id");
   lua_pushcfunction(_get_repository_id);
   lua_settable();

   lua_pushobject(table);
   lua_pushstring("_get_interface_name");
   lua_pushcfunction(_get_interface_name);
   lua_settable();

   lua_pushobject(table);
   lua_pushstring("_get_interface");
   lua_pushcfunction(_get_interface_def);
   lua_settable();

   lua_pushobject(table);
   lua_pushstring("_get_corba_ref");
   lua_pushcfunction(_get_corba_ref);
   lua_settable();

   lua_pushobject(table);
   lua_pushstring("_get_ior");
   lua_pushcfunction(_get_ior);
   lua_settable();

   lua_pushobject(table);
   lua_settag(LuaOrb::TAG);

   lua_pushobject(table);
   this->lua_this = lua_ref(0);
}

lua_Object LuaOrb::GetLuaThis()
{
   return lua_getref(this->lua_this);
}

void LuaOrb::SetLuaThis( lua_Object l_obj )
{
   lua_unref(this->lua_this);
   lua_pushobject(l_obj);
   this->lua_this = lua_ref(0);
}

char* LuaOrb::parsename(char* full_name, InvokeMode* mode)
{
   static char prefix [] = "deferred_";
   char* name = full_name;

   if (mode!=NULL)
   {
      unsigned int n = strlen(prefix);

      *mode = NORMAL_CALL;

      if (strlen(full_name) > n)
      {
         if (strncmp(prefix,full_name,n) == 0)
         {
            name = & full_name[n];
            *mode = DEFERRED_CALL;
         }
      }
   }

   return name;
}

lua_Object LuaOrb::GetLuaExceptionHandler()
{
   lua_pushobject(this->GetLuaThis());
   lua_pushstring("exception_handler");
   return lua_gettable();
}

void LuaOrb::RaiseException( CORBA::Exception* excep, MethodDescr* descr)
{
   #ifdef CORBA_2_3
   CORBA::UnknownUserException* u_ex = CORBA::UnknownUserException::_downcast(excep);
   #else
   CORBA::UnknownUserException* u_ex = CORBA::UnknownUserException::_narrow(excep);
   #endif

   lua_Object fn = this->GetLuaExceptionHandler();
   if (u_ex && lua_isfunction(fn))
   {
      LuaOrb::push(this);  //object
      lua_pushstring(descr->name); //the method name
      lua_pushstring((char*)u_ex->exception().type()->name()); //the exception name
      tc2lua(u_ex->exception().type(),&(u_ex->exception())); //exception's fields
      lua_callfunction(fn);

      //to push the results from fn
      int i = 1;
      lua_Object l_obj;
      while (( l_obj = lua_getresult(i)) != LUA_NOOBJECT)
      {
         lua_pushobject(l_obj);
         i++;
      }
   }
   else if (lua_isfunction(fn))
   {
      LuaOrb::push(this);  //object
      lua_pushstring(descr->name); //the method name
      lua_pushstring((char*)exception_name(*excep)); //the exception name
      lua_callfunction(fn);

      //to push the results from fn
      int i = 1;
      lua_Object l_obj;
      while (( l_obj = lua_getresult(i)) != LUA_NOOBJECT)
      {
         lua_pushobject(l_obj);
         i++;
      }
   }
   else
   {
      char msg[100];
      sprintf(msg,"LuaOrb Error: The method %s raises an exception %s",
              descr->name, exception_name(*excep));
      throw Err_General(msg);
   }
}

void LuaOrb::Call(MethodDescr* descr, InvokeMode mode, LuaObjList* list)
{
   /* Build the parameter list
      Check the request mode: oneway or not
      Perform the request
      Handle the result, if there is one
      Check if there is an exception
   */
   CORBA::Request_ptr req = this->orb_object->_request(descr->name);
   CORBA::NVList_ptr arguments = req->arguments();

   descr->marshal_params(arguments, list); /* build the parameter list */

   if (descr->orb_descr->exceptions.length() >= 1)
   {
      unsigned int i;
      for (i = 0; i < descr->orb_descr->exceptions.length(); i++)
         req->exceptions()->add( descr->orb_descr->exceptions[i].type);
   }

   if (descr->orb_descr->mode == CORBA(OP_NORMAL))
   {
      CORBA::Any* resultAny = descr->mount_result(req);

      if (mode == NORMAL_CALL)
      {
         req->invoke();   /*method request*/

         if (req->env()->exception() == NULL)
         {
            //it will push the result for Lua
            descr->unmarshal_result(resultAny,arguments);
         }
         else
            this->RaiseException(req->env()->exception(),descr);
      }
      else
      {
         req->send_deferred();
         LuaOrb::DeferredClosure* closure = new LuaOrb::DeferredClosure;
         closure->object = this;
         closure->req = req;
         closure->resultAny = resultAny;
         closure->arguments = arguments;
         closure->method_descr = descr;
         LuaOrb::DefClosureWrapper->push(closure);
         return;
      }
   }
   else
      req->send_oneway(); /*method request*/

   CORBA_release(req);
}

void LuaOrb::SetAttr(AttribDescr* descr, InvokeMode mode, lua_Object l_obj)
{
   char req_name[70];
   sprintf(req_name,"_set_%s",descr->name);

   CORBA::Request_ptr req = this->orb_object->_request(req_name);
   CORBA::NVList_ptr arguments = req->arguments();

   descr->marshal_params(arguments, l_obj); /* build the parameter list */
   if (mode == NORMAL_CALL)
      req->invoke();   /*method request*/
   else
   {
      req->send_deferred();

      LuaOrb::DeferredClosure* closure = new LuaOrb::DeferredClosure;
      closure->object = this;
      closure->req = req;
      closure->attrib_descr = descr;
      LuaOrb::DefClosureWrapper->push(closure);
      return;
   }
   CORBA_release(req);
}

void LuaOrb::GetAttr(AttribDescr* descr, InvokeMode mode)
{
   char req_name[70];
   sprintf(req_name,"_get_%s",descr->name);
   CORBA::Request_ptr req = this->orb_object->_request(req_name);

   CORBA::Any* resultAny = descr->mount_result(req);

   if (mode == NORMAL_CALL)
   {
      req->invoke();   /*method request*/

      if (resultAny)
         descr->unmarshal_result(resultAny);   //it will push the result for Lua
   }
   else
   {
      req->send_deferred();
      LuaOrb::DeferredClosure* closure = new LuaOrb::DeferredClosure;
      closure->object = this;
      closure->req = CORBA::Request::_duplicate(req);
      closure->resultAny = resultAny;
      closure->attrib_descr = descr;
      LuaOrb::DefClosureWrapper->push(closure);
      return;
   }
   CORBA_release(req);
}


/******************************************************************************/

/**************************** Class LuaOrb::DeferredClosure  *****************/

int LuaOrb::DeferredClosure::Finished()
{
   try
   {
      if (_finished)
      {
         return 1;
      }
      else if (req != NULL && req->poll_response())
      {
         _finished = 1;
         #ifndef ORB_ORBACUS
         req->get_response(); //it is necessary for Visibroker 3.3
         #endif
         return 1;
      }
      else
         return 0;
   }
   catch(CORBA::Exception& excep)
   {
      if (method_descr)
         cout << "Error polling the method " << method_descr->name << endl;
      else
         cout << "Error polling the attribute " << attrib_descr->name << endl;
      cout << excep << endl;
   }
   return 0;
}

void LuaOrb::DeferredClosure::Wait()
{
   try
   {
      if (!_finished && req != NULL)
      {
         req->get_response();
         _finished = 1;
      }
   }
   catch(CORBA::Exception& excep)
   {
      if (method_descr)
         cout << "Error polling the method " << method_descr->name << endl;
      else
         cout << "Error polling the attribute " << attrib_descr->name << endl;
      cout << excep << endl;
   }

}
void LuaOrb::DeferredClosure::GetResult()
{
   if (_finished && req != NULL)
   {
      if (req->env()->exception())
         object->RaiseException(req->env()->exception(),method_descr);
      else if (resultAny || arguments)
      {
         if (method_descr)
            // map_result will push the result for Lua
            method_descr->unmarshal_result(resultAny,arguments);
         else
            // map_result will push the result for Lua
            attrib_descr->unmarshal_result(resultAny);
      }
      else
         lua_pushnil();
   }
   else
      lua_pushnil();
}

/***************************************************************************/
/*                                                                         */
/*  LuaOrb Proxy Constructor                                               */
/*                                                                         */
/****************************************************************************/

static void _createproxy(void)
{
   lua_Object table = luaL_tablearg(1);
   lua_Object l_id, l_interface, l_name, l_ior, l_obj;
   char* id = NULL;
   char* interfacename = NULL;
   char* ior = NULL;
   CORBA::Object_ptr orb_obj = CORBA::Object::_nil();
   LuaOrb* proxy = NULL;

   lua_pushobject(table);
   lua_pushstring("id");
   l_id = lua_gettable();

   lua_pushobject(table);
   lua_pushstring("interface");
   l_interface = lua_gettable();

   lua_pushobject(table);
   lua_pushstring("name");
   l_name = lua_gettable();

   lua_pushobject(table);
   lua_pushstring("ior");
   l_ior = lua_gettable();

   lua_pushobject(table);
   lua_pushstring("object_ref");
   l_obj = lua_gettable();

   if (lua_isstring(l_id))
      id = lua_getstring(l_id);
   if (lua_isstring(l_interface))
      interfacename = lua_getstring(l_interface);

   if (lua_isuserdata(l_obj))
   {
      orb_obj = CORBA::Object::_duplicate((CORBA::Object_ptr) lua_getuserdata(l_obj));
   }
   else if (lua_isstring(l_ior))
   {
      ior = lua_getstring(l_ior);
      try
      {
         orb_obj = LuaOrbCfg::orb->string_to_object(ior);
      }
      catch (CORBA::Exception& e)
      {
         char msg[256];
         sprintf(msg,"Invalid IOR: %s",exception_name(e));
         lua_error(msg);
      }
   }
#ifdef LO_HAS_OSAGENT
   else if (id != NULL)
   {
      char* name = NULL;
      if (lua_isstring(l_name))
         name = lua_getstring(l_name);
      orb_obj = LuaOrbCfg::orb->bind(id,name);
   }
   else if (interfacename != NULL)
   {
      char* name = NULL;
      if (lua_isstring(l_name))
         name = lua_getstring(l_name);
      orb_obj = LuaOrbCfg::orb->bind(interfacename,name);
   }
#endif
   else
   {
      lua_pushnil();
      return;
   }

   if (!CORBA::is_nil(orb_obj))
   {
      try {
         if (LuaOrbCfg::verbose) dump_info(orb_obj);
         proxy = LuaOrb::new_obj(id,interfacename,orb_obj);
      }
      catch (...)
      {
         lua_error("LuaOrb cannot create a proxy.");
      }
   }

   if (proxy != NULL)
      LuaOrb::push(proxy);
   else
      lua_pushnil();
}

/***************************************************************************/
/*                                                                         */
/*  Tag-Methods and Functions related to LuaOrb Proxies                    */
/*                                                                         */
/****************************************************************************/
static void _callmethod(void)
{
   lua_Object p1 = lua_getparam(1);
   LuaOrb* proxy;
   MethodDescr* method_descr;
   InvokeMode mode;
   unsigned int i = 3;
   LuaObjList* list = NULL;

   try
   {
   if (lua_isuserdata(p1))
   {
      i = 4;
      method_descr = (MethodDescr*) lua_getuserdata(p1);
      mode = (InvokeMode) (int) lua_getnumber(lua_getparam(2));
      proxy = LuaOrb::check_tag(3);
   }
   else
   {
      i = 3;
      char* method_name = luaL_check_string(1);
      proxy = LuaOrb::check_tag(2);
      method_name = LuaOrb::parsename(method_name,&mode);
      method_descr = proxy->myclass->get_opdef(method_name);
      if (method_descr == NULL)
      {
         char msg[256];
         sprintf(msg,"%s is not a method of %s",method_name,
                 proxy->myclass->interface_name());
         lua_error(msg);
         return;
      }
   }

   lua_Object param = lua_getparam(i);
   list = new LuaObjList;
   
   while (param != LUA_NOOBJECT )
   {
      list->add(param);
      i ++;
      param = lua_getparam(i);
   }

   proxy->Call(method_descr, mode, list);
   delete list;
   }
   catch (Err_General& excep)
   {
      if (list)
         delete list;
      lua_error(excep.msg);
   }
   catch (Err_NParams& excep)
   {
      delete list;

      char msg[256];
      sprintf(msg,"%s: invalid number of parameters\n %i arguments and %i formal params",
              method_descr->name,excep.args,excep.formal);
      lua_error(msg);
   }
   catch (Err_Param& excep)
   {
      delete list;

      char msg[256];
      sprintf(msg,"%s: the argument %d couldn't be coerced to %s",
              method_descr->name,excep.index,excep.type);
      lua_error(msg);
   }
   catch (Err_NotImpl& excep)
   {
      delete list;

      char msg[256];
      sprintf(msg,"%s: the %s type isn't implemented",method_descr->name,
              excep.type);
      lua_error(msg);
   }
   catch(CORBA::Exception& excep)
   {
      cout << "Error calling the method " << method_descr->name << endl;
      cout << excep << endl;
   }

}

static void index_fb (void)
{
   LuaOrb* proxy = LuaOrb::from_lua(lua_getparam(1));
   lua_Object index = lua_getparam(2);
   InvokeMode mode;

   //the interface repository is required
   if (lua_isstring(index) && proxy != NULL && LuaOrbCfg::orb_ir)
   {
      try
      {
      char* name = lua_getstring(index);
      name = LuaOrb::parsename(name,&mode);
      MethodDescr* descr = proxy->myclass->get_opdef(name);
      if (descr != NULL)
      {
         lua_pushuserdata(descr);
         lua_pushnumber((float)mode);
         lua_pushcclosure(_callmethod,2);
      }
      else
      {
         // it is not an operation
         // now, we will check if it is an attribute
         AttribDescr* descr = proxy->myclass->get_attdef(name);
         if (descr != NULL)
         {
            try
            {
               proxy->GetAttr(descr, mode);
            }
            catch (Err_NotImpl& excep)
            {
               char msg[256];
               sprintf(msg,"attribute %s: the %s type isn't implemented",
                       descr->name,excep.type);
               lua_error(msg);
            }
            catch(CORBA::Exception& excep)
            {
               cout << "Error getting the attribute " << descr->name << endl;
               cout << excep << endl;
            }
         }
         else
         {
            // it does not belong to the interface
            lua_pushnil();
         }
      }
     }
     catch (Err_General& excep)
     {
        lua_error(excep.msg);
     }
   }
   else
      lua_pushnil();
}

static void settable_fb(void)
{
   lua_Object table = lua_getparam(1);
   LuaOrb* proxy = LuaOrb::from_lua(table);
   lua_Object index = lua_getparam(2);
   lua_Object val = lua_getparam(3);
   InvokeMode mode;

   //the interface repository is required
   if (lua_isstring(index) && proxy != NULL && LuaOrbCfg::orb_ir)
   {
      try
      {
      char* attrib_name = lua_getstring(index);
      attrib_name = LuaOrb::parsename(attrib_name, &mode);
      AttribDescr* descr = proxy->myclass->get_attdef(attrib_name);
      if (descr != NULL)
      {
         if (descr->orb_descr->mode == CORBA(ATTR_READONLY))
         {
            char msg[256];
            sprintf(msg,"attribute %s is read-only", descr->name);
            lua_error(msg);
            return;
         }
         try
         {
            proxy->SetAttr(descr, mode, val);
         }
         catch (Err_Param& excep)
         {
            char msg[256];
            sprintf(msg,"attribute %s: couldn't be coerced to %s",
                    descr->name,excep.type);
            lua_error(msg);
         }
         catch (Err_NotImpl& excep)
         {
            char msg[256];
            sprintf(msg,"attribute %s: the %s type isn't implemented",
                    descr->name,excep.type);
            lua_error(msg);
         }
         catch(CORBA::Exception& excep)
         {
            cout << "Error setting the attribute " << descr->name << endl;
            cout << excep << endl;
         }
         return;
      }
      }
      catch (Err_General& excep)
      {
         lua_error(excep.msg);
      }
   }

   lua_pushobject(table);
   lua_pushobject(index);
   lua_pushobject(val);
   lua_rawsettable();
}

static void gc_fb(void)
{
   LuaOrb* proxy = (LuaOrb*) lua_getuserdata(lua_getparam(1));

   if ( ! proxy->locked )
   {
      delete proxy;
      if (LuaOrbCfg::verbose) cout << "object collected" << endl;
   }
}

/***************************************************************************/
/*                                                                         */
/*  Closure's Tag-Methods                                                  */
/*                                                                         */
/****************************************************************************/
static void defclosure_gc_fb(void)
{
   LuaOrb::DeferredClosure* closure = (LuaOrb::DeferredClosure*) lua_getuserdata(lua_getparam(1));
   delete closure;
   if (LuaOrbCfg::verbose) cout << "closure collected" << endl;
}

/***************************************************************************/
/*                                                                         */
/*  Closure's Methods                                                      */
/*                                                                         */
/****************************************************************************/
static void defclosure_finished()
{
   LuaOrb::DeferredClosure* closure = (LuaOrb::DeferredClosure*) LuaOrb::DefClosureWrapper->check_tag(1);
   int result = closure->Finished();
   if (result)
      lua_pushnumber(1.0);
   else
      lua_pushnil();
}

static void defclosure_wait()
{
   LuaOrb::DeferredClosure* closure = (LuaOrb::DeferredClosure*) LuaOrb::DefClosureWrapper->check_tag(1);
   closure->Wait();
}

static void defclosure_getresult()
{
   LuaOrb::DeferredClosure* closure = (LuaOrb::DeferredClosure*) LuaOrb::DefClosureWrapper->check_tag(1);

   if (closure->Finished())
   {
      //It pushs the method's results in the Lua stack
      closure->GetResult();
   }
   else
      lua_error("LuaOrb: deferred call does not finish.");
}


/***************************************************************************/
/*                                                                         */
/*  Predefined Operations of LuaOrb Proxies                                */
/*                                                                         */
/****************************************************************************/
static void _get_ior(void)
{
   LuaOrb* proxy  = LuaOrb::check_tag(1);

   char* ior = LuaOrbCfg::orb->object_to_string(proxy->orb_object);
   lua_pushstring(ior);
   CORBA::string_free(ior);
}

static void _get_repository_id(void)
{
   LuaOrb* proxy  = LuaOrb::check_tag(1);

   if (proxy->myclass->repository_id())
      lua_pushstring(proxy->myclass->repository_id());
   else
      lua_pushnil();
}

static void _get_interface_name(void)
{
   LuaOrb* proxy  = LuaOrb::check_tag(1);

   if (proxy->myclass->interface_name())
      lua_pushstring(proxy->myclass->interface_name());
   else
      lua_pushnil();
}

static void _get_interface_def(void)
{
   LuaOrb* proxy  = LuaOrb::check_tag(1);

   LuaOrb* newproxy = LuaOrb::new_obj(NULL,"CORBA::InterfaceDef",
                                      proxy->myclass->get_interfacedef());
   if (newproxy)
   {
      newproxy->locked = 1;
      LuaOrb::push(newproxy);
   }
   else
      lua_pushnil();
}

static void _get_corba_ref(void)
{
   LuaOrb* proxy  = LuaOrb::check_tag(1);

   lua_pushuserdata(proxy->orb_object);
}

/*****************************************************************************/


static struct luaL_reg functions_tb []= {{"_createproxy",_createproxy},
                                         {"callmethod",_callmethod}};

static struct luaL_reg defclosure_tagmethods_tb[]= {{"gc", defclosure_gc_fb}};

static struct luaL_reg defclosure_methods_tb[]= {{"Finished", defclosure_finished},
                               {"Wait",defclosure_wait},
                               {"GetResult",defclosure_getresult}};

char luaorb_code[] =
"function createproxy (tb)\n"
"   if tb.ior_file then\n"
"      readfrom(tb.ior_file)\n"
"      tb.ior = read()\n"
"      readfrom()\n"
"   end\n"
"   return _createproxy(tb)\n"
"end";

void LuaOrb::open()
{
   LuaOrb::TAG = lua_newtag();
   lua_pushnumber(LuaOrb::TAG);
   lua_setglobal("LUAORB_TAG");

   LuaOrb::UDTAG = lua_newtag();

   lua_pushcfunction(index_fb);
   lua_settagmethod(LuaOrb::TAG,"index");

   lua_pushcfunction(settable_fb);
   lua_settagmethod(LuaOrb::TAG,"settable");

   lua_pushcfunction(gc_fb);
   lua_settagmethod(LuaOrb::UDTAG,"gc");

   LuaOrb::DefClosureWrapper = LuaWrapper::createWrapper(LuaOrb::DEFCLOSURE_TAG);
   LuaOrb::DefClosureWrapper->reg_ud_tagmethods(defclosure_tagmethods_tb,
                         sizeof(defclosure_tagmethods_tb)/sizeof(struct luaL_reg));
   LuaOrb::DefClosureWrapper->reg_methods(defclosure_methods_tb,
                          sizeof(defclosure_methods_tb)/sizeof(struct luaL_reg));


   luaL_openlib(functions_tb,sizeof(functions_tb)/sizeof(struct luaL_reg));
   lua_dostring(luaorb_code);
}

void LuaOrb::close()
{
   if (LuaOrb::DefClosureWrapper)
      delete LuaOrb::DefClosureWrapper;
}

