//*****************************************************************************
//
// LuaOrb Library
// Computer Science Department of PUC-Rio
// Rio de Janeiro, Brazil
//
// $Id: lo_tcode.cpp,v 1.3 2000/10/18 20:01:25 rcerq Exp $
//
//*****************************************************************************


#include <orb_inc.h>

extern "C"
{
#include <lua.h>
#include <lauxlib.h>
}

#include "lo_tcode.h"
#include "lua2tk.h"

int LuaTypeCode::TAG = 0;
int LuaTypeCode::UDTAG = 0;

static char* enum_tk[]={"tk_null",    "tk_void",     "tk_short", "tk_long",
                        "tk_ushort",  "tk_ulong",    "tk_float", "tk_double",
                        "tk_boolean", "tk_char",     "tk_octet", "tk_any",
                        "tk_TypeCode","tk_Principal","tk_objref","tk_struct",
                        "tk_union",   "tk_enum",     "tk_string","tk_sequence",
                        "tk_array",   "tk_alias",    "tk_except"};

void ltc_equal(void)
{
   LuaTypeCode* ltc  =  LuaTypeCode::check_tag(1);
   LuaTypeCode* ltc2 = LuaTypeCode::check_tag(2);
   CORBA::Boolean ret = ltc->get_tc()->equal(ltc2->get_tc());
   if (ret)
      lua_pushnumber(1.0F);
   else
      lua_pushnil();
}

void ltc_kind(void)
{
   LuaTypeCode* ltc = LuaTypeCode::check_tag(1);
   lua_pushstring( enum_tk[ltc->get_tc()->kind()] );
}

void ltc_id(void)
{
   LuaTypeCode* ltc = LuaTypeCode::check_tag(1);
   enum CORBA(TCKind) tk = ltc->get_tc()->kind();

   if (tk == CORBA(tk_objref) || tk == CORBA(tk_struct) ||
       tk == CORBA(tk_union)  || tk == CORBA(tk_enum)   ||
       tk == CORBA(tk_alias)  || tk == CORBA(tk_except))
      lua_pushstring((char*) ltc->get_tc()->id());
   else
      lua_pushnil();
}

void ltc_name(void)
{
   LuaTypeCode* ltc = LuaTypeCode::check_tag(1);
   enum CORBA(TCKind) tk = ltc->get_tc()->kind();

   if (tk == CORBA(tk_objref) || tk == CORBA(tk_struct) ||
       tk == CORBA(tk_union)  || tk == CORBA(tk_enum)   ||
       tk == CORBA(tk_alias)  || tk == CORBA(tk_except))
      lua_pushstring((char*) ltc->get_tc()->name());
   else
      lua_pushnil();
}

void ltc_member_count(void)
{
   LuaTypeCode* ltc = LuaTypeCode::check_tag(1);
   enum CORBA(TCKind) tk = ltc->get_tc()->kind();

   if (tk == CORBA(tk_struct) || tk == CORBA(tk_union) ||
       tk == CORBA(tk_enum)   || tk == CORBA(tk_except))
      lua_pushnumber((float) ltc->get_tc()->member_count());
   else
      lua_pushnil();
}

void ltc_member_name(void)
{
   LuaTypeCode* ltc = LuaTypeCode::check_tag(1);
   unsigned long index = (unsigned long) luaL_check_number(2);

   enum CORBA(TCKind) tk = ltc->get_tc()->kind();

   if (tk == CORBA(tk_struct) || tk == CORBA(tk_union) ||
       tk == CORBA(tk_enum)   || tk == CORBA(tk_except))
      lua_pushstring((char*) ltc->get_tc()->member_name(index));
   else
      lua_pushnil();
}

void ltc_member_type(void)
{
   LuaTypeCode* ltc = LuaTypeCode::check_tag(1);
   unsigned long index = (unsigned long) luaL_check_number(2);

   enum CORBA(TCKind) tk = ltc->get_tc()->kind();

   if (tk == CORBA(tk_struct) || tk == CORBA(tk_union) ||
       tk == CORBA(tk_except))
      LuaTypeCode::push(new LuaTypeCode(ltc->get_tc()->member_type(index)));
   else
      lua_pushnil();
}

void ltc_member_label(void)
{
   LuaTypeCode* ltc = LuaTypeCode::check_tag(1);
   unsigned long index = (unsigned long) luaL_check_number(2);

   enum CORBA(TCKind) tk = ltc->get_tc()->kind();

   if (tk == CORBA(tk_union))
   {
      CORBA::Any* any = ltc->get_tc()->member_label(index);
      tc2lua(any->type(),any);
   }
   else
      lua_pushnil();
}

void ltc_discriminator_type(void)
{
   LuaTypeCode* ltc = LuaTypeCode::check_tag(1);
   enum CORBA(TCKind) tk = ltc->get_tc()->kind();

   if (tk == CORBA(tk_union))
      LuaTypeCode::push(new LuaTypeCode(ltc->get_tc()->discriminator_type()));
   else
      lua_pushnil();
}

void ltc_default_index(void)
{
   LuaTypeCode* ltc = LuaTypeCode::check_tag(1);
   enum CORBA(TCKind) tk = ltc->get_tc()->kind();

   if (tk == CORBA(tk_union))
      lua_pushnumber((float) ltc->get_tc()->default_index());
   else
      lua_pushnil();
}

void ltc_length(void)
{
   LuaTypeCode* ltc = LuaTypeCode::check_tag(1);
   enum CORBA(TCKind) tk = ltc->get_tc()->kind();

   if (tk == CORBA(tk_string) || tk == CORBA(tk_sequence) ||
       tk == CORBA(tk_array))
      lua_pushnumber((float) ltc->get_tc()->length());
   else
      lua_pushnil();
}

void ltc_content_type(void)
{
   LuaTypeCode* ltc = LuaTypeCode::check_tag(1);
   enum CORBA(TCKind) tk = ltc->get_tc()->kind();

   if (tk == CORBA(tk_sequence) || tk == CORBA(tk_array) ||
       tk == CORBA(tk_alias))
      LuaTypeCode::push(new LuaTypeCode(ltc->get_tc()->content_type()));
   else
      lua_pushnil();
}

static void gc_fb(void)
{
   LuaTypeCode* ltc = (LuaTypeCode*) lua_getuserdata(lua_getparam(1));

   delete ltc;
}

static void register_methods(lua_Object table, struct luaL_reg* list, int n)
{
   for (int i = 0; i < n; i++)
   {
      lua_pushobject(table);
      lua_pushstring(list[i].name);
      lua_pushcfunction(list[i].func);
      lua_settable();
   }
}

//////////////////////////////////////////////////////////////////////////////

static struct luaL_reg methods_tb [] ={
                           {"equal",              ltc_equal},
                           {"kind",               ltc_kind},
                           {"id",                 ltc_id},
                           {"name",               ltc_name},
                           {"member_count",       ltc_member_count},
                           {"member_name",        ltc_member_name},
                           {"member_type",        ltc_member_type},
                           {"member_label",       ltc_member_label},
                           {"discriminator_type", ltc_discriminator_type},
                           {"default_index",      ltc_default_index},
                           {"length",             ltc_length},
                           {"content_type",       ltc_content_type}
                        };

void LuaTypeCode::open()
{
   LuaTypeCode::TAG = lua_newtag();
   lua_pushnumber(LuaTypeCode::TAG);
   lua_setglobal("LUATYPECODE_TAG");

   LuaTypeCode::UDTAG = lua_newtag();

   lua_pushcfunction(gc_fb);
   lua_settagmethod(LuaTypeCode::UDTAG,"gc");
}

LuaTypeCode::~LuaTypeCode()
{
   CORBA_release(this->corba_tc);
}

void LuaTypeCode::push(LuaTypeCode* ltc)
{
   lua_Object table = lua_createtable();

   lua_pushobject(table);
   lua_pushstring("_CORBA_TYPECODE_REF_");
   lua_pushusertag(ltc,LuaTypeCode::UDTAG);
   lua_settable();

   register_methods(table, methods_tb,
                    sizeof(methods_tb)/sizeof(struct luaL_reg));

   lua_pushobject(table);
   lua_settag(LuaTypeCode::TAG);

   lua_pushobject(table);
}

LuaTypeCode* LuaTypeCode::check_tag(int index)
{
   LuaTypeCode* ltc = LuaTypeCode::from_lua(lua_getparam(index));

   luaL_arg_check((ltc!=NULL),index,"it isn't a CORBA::TypeCode Object");

   return ltc;
}

LuaTypeCode* LuaTypeCode::from_lua(lua_Object luaobj)
{
   if (lua_istable(luaobj) && lua_tag(luaobj) == LuaTypeCode::TAG)
   {
      lua_pushobject(luaobj);
      lua_pushstring("_CORBA_TYPECODE_REF_");
      return (LuaTypeCode*) lua_getuserdata(lua_gettable());
   }
   else
      return NULL;
}

LuaTypeCode::LuaTypeCode( CORBA::TypeCode_ptr tc )
{
   this->corba_tc = CORBA::TypeCode::_duplicate(tc);
}
