//*****************************************************************************
//
// LuaOrb Library
// Computer Science Department of PUC-Rio
// Rio de Janeiro, Brazil
//
// $Id: lua2tk.cpp,v 1.44 2000/10/04 21:29:58 rcerq Exp $
//
//*****************************************************************************


#include <orb_inc.h>

extern "C"
{
#include <lua.h>
}

#include "lua2tk.h"
#include "lo_proxy.h"
#include "lo_cfg.h"
#include "lo_except.h"
#include "lo_compt.h"
#include "lo_tcode.h"
#include "lo_dsi.h"

static void lua2short(lua_Object luaobj, CORBA::Any* anyValue)
{
   if (!lua_isnumber(luaobj))
   {
       throw Err_Param("short");
   }
   short luaval = (short)lua_getnumber(luaobj);

   *anyValue <<= luaval;
}

static void lua2ushort(lua_Object luaobj, CORBA::Any* anyValue)
{
    if (!lua_isnumber(luaobj))
    {
       throw Err_Param("unsigned short");
    }
    unsigned short luaval = (unsigned short)lua_getnumber(luaobj);

    *anyValue <<= luaval;
}

static void lua2long(lua_Object luaobj, CORBA::Any* anyValue)
{
    if (!lua_isnumber(luaobj))
    {
       throw Err_Param("long");
    }
    long luaval = (long)lua_getnumber(luaobj);

    *anyValue <<= luaval;
}

static void lua2ulong(lua_Object luaobj, CORBA::Any* anyValue)
{
    if (!lua_isnumber(luaobj))
    {
       throw Err_Param("unsigned long");
    }
    unsigned long luaval = (unsigned long)lua_getnumber(luaobj);

    *anyValue <<= luaval;
}

static void lua2float(lua_Object luaobj, CORBA::Any* anyValue)
{
    if (!lua_isnumber(luaobj))
    {
       throw Err_Param("float");
    }
    float luaval = (float)lua_getnumber(luaobj);

    *anyValue <<= luaval;
}

static void lua2double(lua_Object luaobj, CORBA::Any* anyValue)
{
    if (!lua_isnumber(luaobj))
    {
       throw Err_Param("double");
    }
    double luaval = (double)lua_getnumber(luaobj);

    *anyValue <<= luaval;
}

static void lua2string(lua_Object luaobj, CORBA::Any* anyValue)
{
    const char* luaval = lua_getstring(luaobj);

    if (luaval == NULL)
    {
       throw Err_Param("string");
    }
    *anyValue <<= luaval;
}

static void lua2boolean(lua_Object luaobj, CORBA::Any* anyValue)
{
    char c;
    if (lua_isnil(luaobj))
        c = 0;
    else 
        c = 1;
    CORBA::Any::from_boolean luaval(c);
    *anyValue <<= luaval;
}

static void lua2char(lua_Object luaobj, CORBA::Any* anyValue)
{
    if (!lua_isstring(luaobj))
    {
       throw Err_Param("char");
    }
    char* str = lua_getstring(luaobj);
    CORBA::Any::from_char luaval(str[0]);
    *anyValue <<= luaval;
}

static void lua2octet(lua_Object luaobj, CORBA::Any* anyValue)
{
    if (!lua_isnumber(luaobj))
    {
       throw Err_Param("octet");
    }
    CORBA::Any::from_octet luaval((char)lua_getnumber(luaobj));
    *anyValue <<= luaval;
}

static void lua2objref(CORBA::TypeCode_ptr tc, lua_Object luaobj, CORBA::Any* anyValue)
{
   try
   {
   CORBA::Object_ptr obj;
   LuaOrb* proxy = LuaOrb::from_lua(luaobj);
   #ifdef CORBA_2_3
   DynamicAny::DynAny_var dany =
      LuaOrbCfg::orb_dynany_factory->create_dyn_any_from_type_code(tc);
   #else
   CORBA::Any any_tmp(tc,NULL);
   DYNANY(DynAny_var) dany = LuaOrbCfg::orb->create_dyn_any(any_tmp);
   #endif


   if (proxy != NULL)
   {
      obj = (CORBA::Object*)(proxy->orb_object);
      dany->insert_reference(obj);
      *anyValue = *((const CORBA::Any*)dany->to_any());

   }
   else if (lua_istable(luaobj))
   {
      try {
         LuaDSI* luadsi = (LuaDSI*) LuaDSI::Wrapper->from_lua(luaobj);
         if (luadsi == NULL)
            luadsi = LuaDSI::Create((char*)tc->name(), (char*)tc->id(), luaobj);

         #ifdef CORBA_2_3
         obj = LuaOrbCfg::poa->servant_to_reference(luadsi);
         #else
         obj = luadsi;
         #endif

         dany->insert_reference(obj);
         *anyValue = *((const CORBA::Any*)dany->to_any());
      }
      catch (Err_General& excep) {
          lua_error(excep.msg);
      }
   }
   else if (lua_isnil(luaobj))
   {
      obj = CORBA::Object::_nil();
      *anyValue <<= obj;
   }
   else
      throw Err_Param("object reference");
   }
   catch(CORBA::Exception& excep)
   {
      cout << excep << endl;
      throw;
   }
}

static void lua2struct(CORBA::TypeCode_ptr tc, lua_Object luaobj, CORBA::Any* anyValue)
{
   unsigned long i;
   CORBA::Any any;
   Compat::Marshal* buf = Compat::create_struct_marshaller(tc);

   for (i =0; i < tc->member_count(); i++)
   {
      lua_beginblock();
      lua_pushobject(luaobj);
      lua_pushstring((char*)tc->member_name(i));
      lua_Object field = lua_gettable();

      try
      {
         lua2tc( tc->member_type(i), field, &any);
      }
      catch (Err_Param&)
      {
         delete buf;
         lua_endblock();
         throw Err_Param("struct");
      }
      buf->put(any);

      lua_endblock();
   }

   buf->to_any(anyValue);
   delete buf;
#if 0
   if (LuaOrbCfg::verbose)
   {
      cout << "-----------------------------------" << endl;
      cout << *anyValue << endl;
      cout << "-----------------------------------" << endl;
   }
#endif
}

static void lua2alias(CORBA::TypeCode_ptr tc, lua_Object luaobj, CORBA::Any* anyValue)
{
   lua2tc(tc->content_type(),luaobj,anyValue);
}

static void lua2sequence(CORBA::TypeCode_ptr tc, lua_Object luaobj, CORBA::Any* anyValue)
{
   unsigned long i, length;
   unsigned long max_len = tc->length();
   CORBA::TypeCode_var c_type = tc->content_type();
   CORBA::Any any;
   LuaObjList* list;
   lua_Object field;
   Compat::Marshal* buf;

   i = 1;
   lua_beginblock();

   lua_pushobject(luaobj);
   lua_pushstring("n");
   lua_Object lua_size = lua_gettable();

   if (!lua_isnil(lua_size) && lua_isnumber(lua_size))
   {
      length = (unsigned long) lua_getnumber(lua_size);
      list = new LuaObjList(length);
      while (i <= length)
      {
         lua_pushobject(luaobj);
         lua_pushnumber((float)i);
         field = lua_gettable();
         list->add(field);
         i ++;
      }
   }
   else
   {
      list = new LuaObjList;
      lua_pushobject(luaobj);
      lua_pushnumber((float)i);
      field = lua_gettable();
      while (! lua_isnil(field))
      {
         list->add(field);
         i ++;
         lua_pushobject(luaobj);
         lua_pushnumber((float)i);
         field = lua_gettable();
      }
      length = list->size();
   }

   if (max_len > 0 && length > max_len)
      length = max_len;

   buf = Compat::create_sequence_marshaller(tc,list->size());

   for (i = 0; i < length; i++)
   {
      lua2tc( c_type, list->get(i), &any);
      buf->put(any);
   }

   lua_endblock();

   buf->to_any(anyValue);
   delete buf;
   delete list;
#if 0
   if (LuaOrbCfg::verbose)
   {
      cout << "-----------------------------------" << endl;
      cout << *anyValue << endl;
      cout << "-----------------------------------" << endl;
   }
#endif
}

static void lua2array(CORBA::TypeCode_ptr tc, lua_Object luaobj, CORBA::Any* anyValue)
{
   unsigned long i;
   unsigned long len = tc->length();
   CORBA::TypeCode_var c_type = tc->content_type();
   CORBA::Any any;
   Compat::Marshal* buf = Compat::create_array_marshaller(tc);

   for (i = 0; i < len; i++)
   {
      lua_beginblock();
      lua_pushobject(luaobj);
      lua_pushnumber((float)i+1);
      lua_Object field = lua_gettable();

      try
      {
         lua2tc( c_type, field, &any);
      }
      catch (Err_Param&)
      {
         delete buf;
         lua_endblock();
         throw Err_Param("array");
      }
      buf->put(any);
      lua_endblock();
   }

   buf->to_any(anyValue);
   delete buf;
#if 0
   if (LuaOrbCfg::verbose)
   {
      cout << "-----------------------------------" << endl;
      cout << *anyValue << endl;
      cout << "-----------------------------------" << endl;
   }
#endif
}

static void lua2any(CORBA::TypeCode_ptr,lua_Object luaobj, CORBA::Any* anyValue)
{
   CORBA::Any any;

   if (lua_isnumber(luaobj))
      //converte para tc_double
      lua2tc(CORBA(_tc_double), luaobj, &any);
   else if (lua_isstring(luaobj))
      //converte para tc_string
      lua2tc(CORBA(_tc_string), luaobj, &any);
   else if (lua_istable(luaobj))
   {
      if (lua_tag(luaobj) == LuaOrb::TAG)
         lua2tc(CORBA(_tc_Object), luaobj, &any);
      else if (lua_tag(luaobj) == LuaTypeCode::TAG)
         lua2tc(CORBA(_tc_TypeCode), luaobj, &any);
      else
      {
         lua_pushobject(luaobj);
         lua_pushstring("typecode");
         lua_Object lua_tc = lua_gettable();

         lua_pushobject(luaobj);
         lua_pushstring("value");
         lua_Object lua_value = lua_gettable();

         if ( !lua_isnil(lua_tc)  &&
              lua_istable(lua_tc) && lua_tag(lua_tc) == LuaTypeCode::TAG)
         {
            LuaTypeCode* ltc = LuaTypeCode::from_lua(lua_tc);
            lua2tc(ltc->get_tc(), lua_value, &any);
         }
         else
            //converte para tc_sequence
            lua2tc(LuaOrbCfg::orb->create_sequence_tc(0,CORBA(_tc_any)),
                   luaobj, &any);
      }
   }
   else
      throw Err_Param("any");

   *anyValue <<= any;
}

static void lua2union(CORBA::TypeCode_ptr tc,lua_Object luaobj, CORBA::Any* anyValue)
{
   unsigned long i;
   CORBA::Any any, *discriminator = NULL;
   Compat::Marshal* buf = Compat::create_union_marshaller(tc);
   int err_flag = 0;

   if (lua_istable(luaobj))
   {
      lua_pushobject(luaobj);
      lua_pushstring("union_field");
      lua_Object l_name = lua_gettable();

      lua_pushobject(luaobj);
      lua_pushstring("union_value");
      lua_Object l_value = lua_gettable();

      if ( !lua_isnil(l_name) && lua_isstring(l_name))
      {
         char* name = lua_getstring(l_name);
         for (i = 0; i < tc->member_count(); i++)
            if (strcmp(name, tc->member_name(i)) == 0)
            {
               lua2tc( tc->member_type(i), l_value, &any );
               buf->put(*tc->member_label(i));
               buf->put(any);
               buf->to_any(anyValue);
               delete buf;
               return;
            }
      }
   }

   //if it's not a union table, to try matching
   for (i =0; i < tc->member_count(); i++)
   {
      err_flag = 0;

      discriminator = tc->member_label(i);
      try
      {
         lua2tc( tc->member_type(i), luaobj, &any);
      }
      catch(Err_Param&)
      {
         err_flag = 1;
      }
      if (err_flag == 0)
         break;
   }

   if (err_flag == 0)
   {
      buf->put(*discriminator);
      buf->put(any);
   }
   else
   {
      delete buf;
      throw Err_Param("union");
   }

   buf->to_any(anyValue);
   delete buf;
}

static void lua2enum(CORBA::TypeCode_ptr tc,lua_Object luaobj, CORBA::Any* anyValue)
{
   unsigned long n = tc->member_count();

#ifdef CORBA_2_3
   DynamicAny::DynAny_var tmp = LuaOrbCfg::orb_dynany_factory->create_dyn_any_from_type_code(tc);
   DYNANY(DynEnum_var) dynany = DynamicAny::DynEnum::_narrow(tmp);
#else
   DYNANY(DynEnum_var) dynany = LuaOrbCfg::orb->create_dyn_enum(tc);
#endif

   if (lua_isnumber(luaobj))
   {
      unsigned long val = (unsigned long)lua_getnumber(luaobj);
      if ( val >= n )
         throw Err_Param("enum");
#ifdef CORBA_2_3
      dynany->set_as_ulong(val);
#else
      dynany->value_as_ulong(val);
#endif
   }
   else if (lua_isstring(luaobj))
   {
      char* member = lua_getstring(luaobj);
      unsigned long i;
      for (i = 0; i < n; i++)
         if (strcmp(member,tc->member_name(i)) == 0)
         {
#ifdef CORBA_2_3
            dynany->set_as_string(member);
#else
            dynany->value_as_string(member);
#endif
            break;
         }
      if (i==n)
         throw Err_Param("enum");
   }
   else
      throw Err_Param("enum");

   *anyValue = *((const CORBA::Any*) dynany->to_any());
}

static void lua2typecode(CORBA::TypeCode_ptr, lua_Object luaobj,
                         CORBA::Any* anyValue)
{
   LuaTypeCode* ltc = LuaTypeCode::from_lua(luaobj);

   if (ltc == NULL)
      throw Err_Param("typecode");

   *anyValue <<= ltc->get_tc();
}

static void lua2principal(lua_Object, CORBA::Any*)
{
   throw Err_NotImpl("principal");
}


void lua2tc(CORBA::TypeCode_ptr tc, lua_Object luaobj, CORBA::Any* anyValue)
{
   switch (tc->kind())
   {
   case CORBA(tk_short):
   {
      if (LuaOrbCfg::verbose) cout << "tk_short" << endl;
      lua2short(luaobj,anyValue);
      break;
   }
   case CORBA(tk_long):
   {
      if (LuaOrbCfg::verbose) cout << "tk_long" << endl;
      lua2long(luaobj,anyValue);
      break;
   }
   case CORBA(tk_ushort):
   {
      if (LuaOrbCfg::verbose) cout << "tk_ushort" << endl;
      lua2ushort(luaobj,anyValue);
      break;
   }
   case CORBA(tk_ulong):
   {
      if (LuaOrbCfg::verbose) cout << "tk_ulong" << endl;
      lua2ulong(luaobj,anyValue);
      break;
   }
   case CORBA(tk_float):
   {
      if (LuaOrbCfg::verbose) cout << "tk_float" << endl;
      lua2float(luaobj,anyValue);
      break;
   }
   case CORBA(tk_double):
   {
      if (LuaOrbCfg::verbose) cout << "tk_double" << endl;
      lua2double(luaobj,anyValue);
      break;
   }
   case CORBA(tk_string):
   {
      if (LuaOrbCfg::verbose) cout << "tk_string" << endl;
      lua2string(luaobj,anyValue);
      break;
   }
   case CORBA(tk_boolean):
   {
      if (LuaOrbCfg::verbose) cout << "tk_boolean" << endl;
      lua2boolean(luaobj,anyValue);
      break;
   }
   case CORBA(tk_char):
   {
      if (LuaOrbCfg::verbose) cout << "tk_char" << endl;
      lua2char(luaobj,anyValue);
      break;
   }
   case CORBA(tk_octet):
   {
      if (LuaOrbCfg::verbose) cout << "tk_octet" << endl;
      lua2octet(luaobj,anyValue);
      break;
   }
   case CORBA(tk_objref):
      if (LuaOrbCfg::verbose) cout << "tk_objref" << endl;
      lua2objref(tc,luaobj,anyValue);
      break;
   case CORBA(tk_struct):
      if (LuaOrbCfg::verbose) cout << "tk_struct" << endl;
      lua2struct(tc,luaobj,anyValue);
      break;
   case CORBA(tk_any):
      if (LuaOrbCfg::verbose) cout << "tk_any" << endl;
      lua2any(tc,luaobj,anyValue);
      break;
   case CORBA(tk_union):
      if (LuaOrbCfg::verbose) cout << "tk_union" << endl;
      lua2union(tc,luaobj,anyValue);
      break;
   case CORBA(tk_enum):
      if (LuaOrbCfg::verbose) cout << "tk_enum" << endl;
      lua2enum(tc,luaobj,anyValue);
      break;
   case CORBA(tk_sequence):
      if (LuaOrbCfg::verbose) cout << "tk_sequence" << endl;
      lua2sequence(tc,luaobj,anyValue);
      break;
   case CORBA(tk_array):
      if (LuaOrbCfg::verbose) cout << "tk_array" << endl;
      lua2array(tc,luaobj,anyValue);
      break;
   case CORBA(tk_alias):
      if (LuaOrbCfg::verbose) cout << "tk_alias" << endl;
      lua2alias(tc,luaobj,anyValue);
      break;
   case CORBA(tk_except):
      if (LuaOrbCfg::verbose) cout << "tk_except" << endl;
      lua2struct(tc,luaobj,anyValue);
      break;
   case CORBA(tk_TypeCode):
      if (LuaOrbCfg::verbose) cout << "tk_TypeCode" << endl;
      lua2typecode(tc,luaobj,anyValue);
      break;
   case CORBA(tk_Principal):
      if (LuaOrbCfg::verbose) cout << "tk_Principal" << endl;
      lua2principal(luaobj,anyValue);
      break;
   default:
      break;
   }
}

/********************************************************************/

void tc2lua(CORBA::TypeCode_ptr tc, CORBA::Any* resultAny)
{
   // testa tipo do resultado
   // converte resultado para Lua
   // empilha resultado
   switch (tc->kind()) //(resultAny->type()->kind())
   {
     case CORBA(tk_short):
     {
         if (LuaOrbCfg::verbose) cout << "tk_short" << endl;
         short ret;
         (*resultAny) >>= ret;
         lua_pushnumber((float)ret);
         break;
     }
     case CORBA(tk_long):
     {
         if (LuaOrbCfg::verbose) cout << "tk_long" << endl;
         long ret;
         (*resultAny)  >>= ret;
         lua_pushnumber((float)ret);
         break;
     }
     case CORBA(tk_ushort):
     {
         if (LuaOrbCfg::verbose) cout << "tk_ushort" << endl;
         unsigned short ret;
         (*resultAny)  >>= ret;
         lua_pushnumber((float)ret);
         break;
     }
     case CORBA(tk_ulong):
     {
         if (LuaOrbCfg::verbose) cout << "tk_ulong" << endl;
         unsigned long ret;
         (*resultAny)  >>= ret;
         lua_pushnumber((float)ret);
         break;
     }
     case CORBA(tk_float):
     {
         if (LuaOrbCfg::verbose) cout << "tk_float" << endl;
         float ret;
         (*resultAny)  >>= ret;
         lua_pushnumber(ret);
         break;
     }
     case CORBA(tk_double):
     {
         if (LuaOrbCfg::verbose) cout << "tk_double" << endl;
         double ret;
         (*resultAny)  >>= ret;
         lua_pushnumber((float)ret);
         break;
     }
     case CORBA(tk_string):
     {
         if (LuaOrbCfg::verbose) cout << "tk_string" << endl;

         #if defined(CORBA_2_3) && defined(ORB_ORBACUS)
         const char* ret = NULL;
         #else
         char* ret = NULL;
         #endif

         (*resultAny)  >>= ret;

         lua_pushstring((char*)ret);
         break;
     }
     case CORBA(tk_boolean):
     {
         if (LuaOrbCfg::verbose) cout << "tk_boolean" << endl;

         #if defined(CORBA_2_3) && defined(ORB_ORBACUS)
         bool ret;
         #else
         unsigned char ret;
         #endif

         CORBA::Any::to_boolean acox(ret);
         (*resultAny)  >>= acox; //CORBA::Any::to_boolean(ret);
         if (ret)
            lua_pushnumber((float)1.0);
         else
            lua_pushnil();
         break;
     }
     case CORBA(tk_char):
     {
         if (LuaOrbCfg::verbose) cout << "tk_char" << endl;
         CORBA::Char ret;
         CORBA::Any::to_char acox(ret);
         (*resultAny)  >>= acox;
         char str[2];
         str[0] = ret;
         str[1] = 0;
         lua_pushstring(str);
         break;
     }
     case CORBA(tk_octet):
     {
         if (LuaOrbCfg::verbose) cout << "tk_octet" << endl;
         CORBA::Octet ret;
         CORBA::Any::to_octet acox(ret);
         (*resultAny)  >>= acox;
         lua_pushnumber((float)ret);
         break;
     }
     case CORBA(tk_objref):
     {
         if (LuaOrbCfg::verbose)
         {
            cout << "tk_objref" << endl;
            cout << tc->name() << endl;
            cout << tc->id() << endl;
         }
         CORBA::Object_ptr ret;
         CORBA::Any::to_object to_obj(ret);
         (*resultAny)  >>= to_obj;
         if (CORBA::is_nil(ret))
            lua_pushnil();
         else
         {
            #ifdef CORBA_2_3
            if (LuaOrbCfg::servant_optimization)
            {
               try {
                  PortableServer::Servant servant;
                  servant = LuaOrbCfg::poa->reference_to_servant(ret);
                  if (LuaOrbCfg::verbose) cout << "ObjRef->Servant conversion" << endl;
                  LuaDSI* l_dsi = dynamic_cast<LuaDSI*>(servant);
                  if (l_dsi)
                  {
                     if (LuaOrbCfg::verbose) cout <<"Servant->LuaDSI conversion"<< endl;
                     lua_pushobject(l_dsi->GetLuaObject());
                     servant->_remove_ref();
                     break;
                  }
                  servant->_remove_ref();
               }
               catch(CORBA::UserException& excep)
               {
                  if (LuaOrbCfg::verbose) cout <<"Servant optimization failed: " << excep << endl;
               }
            }
            #endif
            LuaOrb::push(LuaOrb::new_obj(tc->id(), tc->name(),
                                         CORBA::Object::_duplicate(ret)));
         }
         break;
     }
     case CORBA(tk_except):
         if (LuaOrbCfg::verbose) cout << "tk_except" << endl;
     case CORBA(tk_struct):
     {
         if (LuaOrbCfg::verbose) cout << "tk_struct" << endl;
         lua_Object table = lua_createtable();
         unsigned long i;
         CORBA::Any_var any;
         Compat::Unmarshal* buf;
         buf = Compat::create_struct_unmarshaller(tc,*resultAny);

         for (i = 0; i < tc->member_count(); i++)
         {
            any = buf->get();
            lua_pushobject(table);
            lua_pushstring((char*)tc->member_name(i));
            tc2lua(tc->member_type(i),any);
            lua_settable();
         }

         delete buf;
         lua_pushobject(table);

         break;
     }
     case CORBA(tk_alias):
     {
         if (LuaOrbCfg::verbose) cout << "tk_alias" << endl;

         tc2lua(tc->content_type(),resultAny);
         break;
     }
     case CORBA(tk_sequence):
     {
         if (LuaOrbCfg::verbose) cout << "tk_sequence" << endl;
         lua_Object table = lua_createtable();
         unsigned long i, len;
         CORBA::Any_var any;
         CORBA::TypeCode_var c_type = tc->content_type();
         Compat::Unmarshal* buf;

         buf = Compat::create_sequence_unmarshaller(tc, *resultAny, &len);

         lua_pushobject(table);
         lua_pushstring("n");
         lua_pushnumber((float)len);
         lua_settable();

         for (i = 0; i < len; i++)
         {
            any = buf->get();
            lua_pushobject(table);
            lua_pushnumber((float)i+1);
            tc2lua(c_type,any);
            lua_settable();
         }

         delete buf;
         lua_pushobject(table);

         break;
     }
     case CORBA(tk_array):
     {
         if (LuaOrbCfg::verbose) cout << "tk_array" << endl;
         lua_Object table = lua_createtable();
         unsigned long i, len = tc->length();
         CORBA::Any_var any;
         CORBA::TypeCode_var c_type = tc->content_type();
         Compat::Unmarshal* buf;

         buf = Compat::create_array_unmarshaller(tc,*resultAny);

         lua_pushobject(table);
         lua_pushstring("n");
         lua_pushnumber((float)len);
         lua_settable();

         for (i = 0; i < len; i++)
         {
            any = buf->get();
            lua_pushobject(table);
            lua_pushnumber((float)i+1);
            tc2lua(c_type,any);
            lua_settable();
         }

         delete buf;
         lua_pushobject(table);

         break;
     }
     case CORBA(tk_any):
     {
         if (LuaOrbCfg::verbose) cout << "tk_any" << endl;

        #ifdef LO_VISIBROKER_BUG
         CORBA::Any any;
         *resultAny >>= any;
         tc2lua(any.type(),&any);
        #else
        #if defined(CORBA_2_3) && defined(ORB_ORBACUS)
         const CORBA::Any* any;
        #else
         CORBA::Any* any;
        #endif
         *resultAny >>= any;

         tc2lua(any->type(),(CORBA::Any*)any);
        #endif

         break;
     }
     case CORBA(tk_union):
     {
         if (LuaOrbCfg::verbose) cout << "tk_union" << endl;
         CORBA::Any_var any;
         Compat::Unmarshal* buf;
         buf = Compat::create_union_unmarshaller(tc,*resultAny);
         
         //discard the discriminator
         any = buf->get();
         any = buf->get();
         tc2lua(any->type(),any);

         delete buf;
         break;
     }
     case CORBA(tk_enum):
     {
         if (LuaOrbCfg::verbose) cout << "tk_enum" << endl;

#ifdef CORBA_2_3
        DynamicAny::DynAny_var tmp = LuaOrbCfg::orb_dynany_factory->create_dyn_any(*resultAny);
        DYNANY(DynEnum_var) dynany = DynamicAny::DynEnum::_narrow(tmp);
#else
        DYNANY(DynEnum_var) dynany = LuaOrbCfg::orb->create_dyn_enum(tc);
        dynany->from_any(*resultAny);
#endif

         if (LuaOrbCfg::enum_numeric_mode)
#ifdef CORBA_2_3
            lua_pushnumber((float)dynany->get_as_ulong());
#else
            lua_pushnumber((float)dynany->value_as_ulong());
#endif
         else
#ifdef CORBA_2_3
            lua_pushstring((char*)dynany->get_as_string());
#else
            lua_pushstring((char*)dynany->value_as_string());
#endif
         break;
     }
     case CORBA(tk_TypeCode):
     {
         if (LuaOrbCfg::verbose) cout << "tk_TypeCode" << endl;
         CORBA::TypeCode_ptr atc;
         *resultAny >>= atc;
         LuaTypeCode* ltc = new LuaTypeCode(atc);
         LuaTypeCode::push(ltc);
         break;
     }
     case CORBA(tk_Principal):
         if (LuaOrbCfg::verbose) cout << "tk_Principal" << endl;
         break;
     default:
         break;
   }
}
