//*****************************************************************************
//
// LuaOrb Library
// Computer Science Department of PUC-Rio
// Rio de Janeiro, Brazil
//
// $Id: luawrap.cpp,v 1.2 1999/06/10 22:06:15 rcerq Exp $
//
//*****************************************************************************

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
extern "C"
{
#include <lua.h>
#include <lauxlib.h>
}
#include "luawrap.h"

int LuaWrapper::initialized = 0;
char* LuaWrapper::init_code = "LuaWrapperTable = {}";

static void register_methods(lua_Object table, struct luaL_reg* list, int n)
{
   for (int i = 0; i < n; i++)
   {
      lua_pushobject(table);
      lua_pushstring(list[i].name);
      lua_pushcfunction(list[i].func);
      lua_settable();
   }
}

static void register_tagmethods(int tag, struct luaL_reg* table, int n)
{
   int i = 0;
   for (i=0; i < n; i++)
   {
      lua_pushcfunction(table[i].func);
      lua_settagmethod(tag,table[i].name);
   }
}

LuaWrapper* LuaWrapper::createWrapper(const char* name)
{
   if ( !LuaWrapper::initialized )
   {
      lua_dostring(LuaWrapper::init_code);
      LuaWrapper::initialized = 1;
   }

   LuaWrapper* lwrap = new LuaWrapper(name);
   
   lua_pushobject(lua_getglobal("LuaWrapperTable"));
   lua_pushstring((char*)name);
   lua_pushuserdata(lwrap);
   lua_settable();

   return lwrap;
}

LuaWrapper* LuaWrapper::getWrapper(const char* name)
{
   lua_pushobject(lua_getglobal("LuaWrapperTable"));
   lua_pushstring((char*)name);
   lua_Object lobj = lua_gettable();
   LuaWrapper* lwrap = (LuaWrapper*) lua_getuserdata(lobj);

   return lwrap;
}

LuaWrapper::LuaWrapper( const char* name )
{
   char lua_name[50];

   this->mlist = NULL;

   sprintf(lua_name,"LB_%s_TAG",name);
   this->tag_name = strdup(lua_name);

   sprintf(lua_name,"LB_%s_UDTAG",name);

   this->tag = lua_newtag();
   lua_pushnumber((float)tag);
   lua_setglobal(tag_name);

   this->ud_tag = lua_newtag();
   lua_pushnumber((float)ud_tag);
   lua_setglobal(lua_name);
}

LuaWrapper::~LuaWrapper()
{
   free(this->tag_name);
   if (this->mlist)
      delete this->mlist;
}

void LuaWrapper::reg_tagmethods(struct luaL_reg* table, int n)
{
   register_tagmethods(this->tag,table,n);
}

void LuaWrapper::reg_ud_tagmethods(struct luaL_reg* table, int n)
{
   register_tagmethods(this->ud_tag,table,n);
}

void LuaWrapper::reg_methods(struct luaL_reg* table, int n)
{
   if (this->mlist == NULL)
   {
      this->mlist = new LuaWrapper::MethodsList;
   }
   this->mlist->n = n;
   this->mlist->table = table;
}

void LuaWrapper::push( void* userdata )
{
   lua_Object table = lua_createtable();

   lua_pushobject(table);
   lua_pushstring("_USERDATA_REF_");
   lua_pushusertag(userdata,this->ud_tag);
   lua_settable();

   if (this->mlist)
      register_methods(table, this->mlist->table, this->mlist->n);

   lua_pushobject(table);
   lua_settag(this->tag);

   lua_pushobject(table);
}

void* LuaWrapper::check_tag(int index)
{
   void* userdata = this->from_lua(lua_getparam(index));

   luaL_arg_check((userdata!=NULL),index,"Object type is wrong");

   return userdata;
}

void* LuaWrapper::from_lua(lua_Object luaobj)
{
   if (lua_istable(luaobj) && lua_tag(luaobj) == this->tag)
   {
      lua_pushobject(luaobj);
      lua_pushstring("_USERDATA_REF_");
      return lua_getuserdata(lua_gettable());
   }
   else
      return NULL;
}

