//*****************************************************************************
//
// LuaOrb Library
// Computer Science Department of PUC-Rio
// Rio de Janeiro, Brazil
//
// $Id: orb_inc.h,v 1.25 2000/10/18 20:01:25 rcerq Exp $
//
//*****************************************************************************


#ifndef __ORB_INC_H
#define __ORB_INC_H


#ifdef CORBA_2_3

# ifdef ORB_ORBACUS

#include <OB/CORBA.h>
#include <OB/IFR.h>
#include <OB/Properties.h>

#ifdef HAVE_IOSTREAM
#include <iostream>
using namespace std;
#else
#include <iostream.h>
#endif

#define OB_MAKE_NARROW_IMPL(x)
#define OB_MAKE_NARROW_IMPL_1(x,y)

# else // ORB_MICO

#include <CORBA.h>

# endif


inline const char* exception_name(CORBA::Exception& ex)
{
   CORBA::Any tmp;
   tmp <<= ex;
   CORBA::TypeCode_var tc = tmp.type();
   const char* p = tc->name();
   if (*p != '\0')
      return p;
   else
      return tc->id();
}


inline void dump_info(CORBA::Object_ptr obj)
{
   try
   {
   CORBA::InterfaceDef_var idef = obj->_get_interface();
   if(CORBA::is_nil(idef))
   {
       cout << "LuaOrb: no interface repository available" << endl;
   }
   else
   {
      CORBA::InterfaceDef::FullInterfaceDescription_var desc = idef -> describe_interface();

      cout << "name = " << desc -> name << endl;
      cout << "id = " << desc -> id << endl;
      cout << "version = " << desc -> version << endl;
   }
   }
   catch(CORBA::SystemException& ex)
   {
      cout << "Got an exception: " << ex << endl;
   }
}


#define CORBA(x) CORBA:: ## x
#define CORBAEXP(x) CORBA:: ## x
#define DYNANY(x) DynamicAny:: ## x


inline void CORBA_release(CORBA::Object_ptr obj)
{
   CORBA::release(obj);
}

inline void CORBA_release(CORBA::Request_ptr obj)
{
   CORBA::release(obj);
}

inline void CORBA_release(CORBA::TypeCode_ptr obj)
{
   CORBA::release(obj);
}

inline void CORBA_release(CORBA::ContainedSeq* seq)
{
   delete seq;
}


#else // versao < CORBA_2_3

#ifdef ORB_ORBACUS
#include <OB/CORBA.h>
#include <OB/Util.h>

#ifdef HAVE_IOSTREAM
#include <iostream>
using namespace std;
#else
#include <iostream.h>
#endif

#define CORBA(x) CORBA_ ## x
#define CORBAEXP(x) CORBA_ ## x
#define DYNANY(x) CORBA:: ## x

#ifndef GCC_2_7
  #define OB_MAKE_NARROW_IMPL(x)
  #define OB_MAKE_NARROW_IMPL_1(x,y)
#else
  #include <OB/Narrow_impl.h>
#endif

class CORBA
{
   public:

   typedef CORBA_Long Long;
   typedef CORBA_ULong ULong;
   typedef CORBA_Short Short;
   typedef CORBA_UShort UShort;
   typedef CORBA_Float Float;
   typedef CORBA_Double Double;
   typedef CORBA_Char Char;
   typedef CORBA_Boolean Boolean;

   typedef CORBA_Flags Flags;

   typedef CORBA_String_var String_var;

   typedef CORBA_DynAny DynAny;
   typedef CORBA_DynAny_ptr DynAny_ptr;
   typedef CORBA_DynAny_var DynAny_var;

   typedef CORBA_DynUnion DynUnion;
   typedef CORBA_DynUnion_ptr DynUnion_ptr;
   typedef CORBA_DynUnion_var DynUnion_var;

   typedef CORBA_DynStruct DynStruct;
   typedef CORBA_DynStruct_ptr DynStruct_ptr;
   typedef CORBA_DynStruct_var DynStruct_var;

   typedef CORBA_DynSequence DynSequence;
   typedef CORBA_DynSequence_ptr DynSequence_ptr;
   typedef CORBA_DynSequence_var DynSequence_var;

   typedef CORBA_DynArray DynArray;
   typedef CORBA_DynArray_ptr DynArray_ptr;
   typedef CORBA_DynArray_var DynArray_var;

   typedef CORBA_DynEnum DynEnum;
   typedef CORBA_DynEnum_ptr DynEnum_ptr;
   typedef CORBA_DynEnum_var DynEnum_var;

   typedef CORBA_Any Any;
   typedef CORBA_Any_var Any_var;

   typedef CORBA_AttributeDef AttributeDef;
   typedef CORBA_AttributeDef_ptr AttributeDef_ptr;
   typedef CORBA_AttributeDef_var AttributeDef_var;

   typedef CORBA_InterfaceDef InterfaceDef;
   typedef CORBA_InterfaceDef_ptr InterfaceDef_ptr;
   typedef CORBA_InterfaceDef_var InterfaceDef_var;

   typedef CORBA_AttributeMode AttributeMode;
   typedef CORBA_OperationMode OperationMode;

   typedef CORBA_Contained Contained;
   typedef CORBA_Contained_ptr Contained_ptr;
   typedef CORBA_Contained_var Contained_var;
   typedef CORBA_ContainedSeq ContainedSeq;

   typedef CORBA_Exception Exception;
   //typedef CORBA_Exception_ptr Exception_ptr;
   //typedef CORBA_Exception_var Exception_var;

   typedef CORBA_SystemException SystemException;
   typedef CORBA_UnknownUserException UnknownUserException;
   typedef CORBA_ExceptionDefSeq     ExceptionDefSeq;

   typedef CORBA_IDLType IDLType;
   typedef CORBA_IDLType_ptr IDLType_ptr;
   typedef CORBA_IDLType_var IDLType_var;

   typedef CORBA_NVList NVList;
   typedef CORBA_NVList_ptr NVList_ptr;
   typedef CORBA_NVList_var NVList_var;

   typedef CORBA_NamedValue NamedValue;
   typedef CORBA_NamedValue_var NamedValue_var;
   typedef CORBA_NamedValue_ptr NamedValue_ptr;

   typedef CORBA_ORB ORB;
   typedef CORBA_ORB_ptr ORB_ptr;
   typedef CORBA_ORB_var ORB_var;

   typedef CORBA_BOA BOA;
   typedef CORBA_BOA_ptr BOA_ptr;
   typedef CORBA_BOA_var BOA_var;

   typedef CORBA_Object Object;
   typedef CORBA_Object_ptr Object_ptr;
   typedef CORBA_Object_var Object_var;

   typedef CORBA_Octet Octet;

   typedef CORBA_OperationDef OperationDef;
   typedef CORBA_OperationDef_ptr OperationDef_ptr;
   typedef CORBA_OperationDef_var OperationDef_var;

   typedef CORBA_ParDescriptionSeq ParDescriptionSeq;
   //typedef CORBA_ParDescriptionSeq_ptr ParDescriptionSeq_ptr;
   typedef CORBA_ParDescriptionSeq_var ParDescriptionSeq_var;

   typedef CORBA_Repository Repository;
   typedef CORBA_Repository_ptr Repository_ptr;
   typedef CORBA_Repository_var Repository_var;

   typedef CORBA_Request Request;
   typedef CORBA_Request_ptr Request_ptr;
   typedef CORBA_Request_var Request_var;

   typedef CORBA_TCKind TCKind;

   typedef CORBA_TypeCode TypeCode;
   typedef CORBA_TypeCode_ptr TypeCode_ptr;
   typedef CORBA_TypeCode_var TypeCode_var;

   typedef CORBA_DynamicImplementation DynamicImplementation;

   typedef CORBA_ServerRequest ServerRequest;
   typedef CORBA_ServerRequest_ptr ServerRequest_ptr;
   typedef CORBA_ServerRequest_var ServerRequest_var;

   typedef CORBA_OperationDescription OperationDescription;
   typedef CORBA_OperationDescription_var OperationDescription_var;
   typedef CORBA_AttributeDescription AttributeDescription;
   typedef CORBA_AttributeDescription_var AttributeDescription_var;
   typedef CORBA_ParameterDescription ParameterDescription;

   static CORBA_Boolean is_nil(CORBA_InterfaceDef_ptr obj)
   {   return CORBA_is_nil(obj); }
   static CORBA_Boolean is_nil(CORBA_Object_ptr obj)
   {   return CORBA_is_nil(obj); }
   static CORBA_Boolean is_nil(CORBA_TypeCode_ptr obj)
   {   return CORBA_is_nil(obj); }

   static char* string_dup( const char* str )
   { return CORBA_string_dup(str); }
   static void string_free( char* str )
   { CORBA_string_free(str); }
};

inline ostream& operator << (ostream& stream,
                             const CORBA_Exception& except)
{
   return stream << except._type_id();
}

inline const char* exception_name(CORBA::Exception& ex)
{
   return ex._type_id();
}

inline void dump_info(CORBA::Object_ptr obj)
{
   try
   {
   CORBA_InterfaceDef_var idef = obj->_get_interface();
   if(CORBA_is_nil(idef))
   {
       cout << "LuaOrb: no interface repository available" << endl;
   }
   else
   {
      CORBA_InterfaceDef::FullInterfaceDescription_var desc = idef -> describe_interface();

      cout << "name = " << desc -> name << endl;
      cout << "id = " << desc -> id << endl;
      cout << "version = " << desc -> version << endl;
   }
   }
   catch(CORBA_SystemException& ex)
   {
      OBPrintException(ex);
   }
}

inline void CORBA_release(CORBA::ContainedSeq* seq)
{
   delete seq;
}

#else //ORB_VISIBROKER

#include <corba.h>

#define CORBA(x) CORBA:: ## x
#define CORBAEXP(x) CORBA_ ## x
#define DYNANY(x) CORBA:: ## x


inline void CORBA_release(CORBA::Object_ptr obj)
{
   obj->_release();
}

inline void CORBA_release(CORBA::Request_ptr obj)
{
   CORBA::Request::_release(obj);
}

inline void CORBA_release(CORBA::TypeCode_ptr obj)
{
   CORBA::TypeCode::_release(obj);
}

inline void CORBA_release(CORBA::ContainedSeq* seq)
{
      CORBA::ContainedSeq::_release(seq);
}

inline const char* exception_name(CORBA::Exception& ex)
{
   return ex._name();
}

inline void dump_info(CORBA::Object_ptr obj)
{
   cout << obj->_interface_name() << endl;
   cout << obj->_repository_id() << endl;
}

inline void operator >>= (CORBA::Any& any, CORBA::OperationDescription*& descr)
{
   descr = new CORBA::OperationDescription;
   //any >>= *descr;
   CORBA::MarshalInBuffer _mbuf((char*)any.value(),(CORBA::ULong)any.len());
   _mbuf >> *descr;
}

inline void operator >>= (CORBA::Any& any, CORBA::AttributeDescription*& descr)
{
   descr = new CORBA::AttributeDescription;
   //any >>= *descr;
   CORBA::MarshalInBuffer _mbuf((char*)any.value(),(CORBA::ULong)any.len());
   _mbuf >> *descr;
}

#endif

#endif /* versao CORBA */

#endif /* __ORB_INC_H */
